<?php
/**
*
* @package arcade
* @version $Id: install.php 363 2008-09-05 13:56:25Z jrsweets $
* @copyright (c) 2008 http://www.JeffRusso.net
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/

define('IN_PHPBB', true);
$phpbb_root_path = (defined('PHPBB_ROOT_PATH')) ? PHPBB_ROOT_PATH : './../';
$phpEx = substr(strrchr(__FILE__, '.'), 1);
require($phpbb_root_path . 'common.' . $phpEx);
require($phpbb_root_path . 'includes/db/db_tools.' . $phpEx);

// Report all errors, except notices
error_reporting(E_ALL);
@set_time_limit(0);

// Start session management
$user->session_begin();
$auth->acl($user->data);
$user->setup();

// is the user logged in?
// This might be looked at later if people have problems, but I guess only the founder
// should be using ftp and installing scripts anyway.
if ($user->data['user_type'] != USER_FOUNDER)
{
	trigger_error('You are not authorized to use this script.<br /><br />Please note to use the script the following requirements must be met.  First you must be logged in to the site and second you must have the user type founder. If you are logged in and are the founder then you have incorrect cookie settings in the acp. Please check the cookie domain setting. If your site url is <strong>http://www.example.com</strong> then the cookie domain should be <strong>.example.com</strong>.');
}

// Check to make sure version 3.0.1 of phpBB is installed
if (version_compare(str_replace('rc', 'RC', strtolower($config['version'])), '3.0.1', '<'))
{
	trigger_error('This version of phpBB Arcade is designed to run on phpBB 3.0.1 or higher.  Please upgrade your phpBB installation before running this version of the phpBB Arcade.');
}

require('./config.' . $phpEx);
$mode = request_var('mode', '');
$db->sql_return_on_error(true);

$install_mod = new install_mod();
$phpbb_db_tools = new phpbb_db_tools($db);
$install_mod->install_header($CFG['mod_title']);

switch ($mode)
{
	case 'uninstall':
		echo '<h1>' . $CFG['mod_title'] . ' Uninstallation</h1>';
		echo '<p>Table Prefix :: ' . $table_prefix . '<br />';
		echo 'Database Type :: ' . $db->sql_layer . '</p>';

		echo '<br /><h3>Removing mod database tables and data...</h3>';
		$install_mod->load_data($CFG['remove_data_file']);
		switch ($db->sql_layer)
		{
			case 'postgres':
				$install_mod->load_data($CFG['pg_remove_data_file']);
			break;
			
			default:
			break;
		}
		
		if (isset($CFG['remove_schema_changes']))
		{
			$phpbb_db_tools->perform_schema_changes($CFG['remove_schema_changes']);
		}

		echo '<br /><h3>Removing mod permissions...</h3>';
		$install_mod->remove_permissions($CFG['permission_options']);

		echo '<br /><h3>Removing mod modules and clearing module cache...</h3>';
		$install_mod->remove_modules($CFG['parent_module_remove'], $CFG['module_remove']);

		echo '<br /><h3>Clearing permission cache...</h3>';
		$install_mod->clear_permission_cache();

		if ($CFG['clear_cache_uninstall'])
		{
			echo '<br /><h3>Clearing cache...</h3>';
			$install_mod->clear_cache();
		}

		echo '<br /><h3>' . $CFG['mod_title'] . ' has been uninstalled</h3><br />';
		echo '<h3>Remove ' . $CFG['mod_dir'] . ' directory from your server</h3>';

		$install_mod->install_footer();
	break;

	case 'install':
		echo '<h1>' . $CFG['mod_title'] . ' Installation</h1>';
		echo '<p>Table Prefix :: ' . $table_prefix . '<br />';
		echo 'Database Type :: ' . $db->sql_layer . '</p>';

		echo '<br /><h3>Creating mod database tables...</h3>';
		$install_mod->load_tables();

		echo '<br /><h3>Inserting mod data...</h3>';
		// Ok tables have been built, let's fill in the basic information
		$install_mod->load_data($CFG['data_file']);
		if (isset($CFG['schema_changes']))
		{
			$phpbb_db_tools->perform_schema_changes($CFG['schema_changes']);
		}

		echo '<br /><h3>Inserting mod permissions and clearing permission cache...</h3>';
		// Add permissions
		$install_mod->add_permissions($CFG['permission_options']);

		echo '<br /><h3>Inserting mod modules and clearing module cache...</h3>';
		// Create Mod Modules...
		foreach($CFG['mod_modules'] as $modules)
		{
			$install_mod->create_modules($modules['parent_module_data'], $modules['module_data']);
		}

		if ($CFG['clear_cache_install'])
		{
			echo '<br /><h3>Clearing cache...</h3>';
			$install_mod->clear_cache();
		}

		echo '<br /><h3>' . $CFG['mod_title'] . ' has been installed</h3><br />';
		echo '<h3>Remove ' . $CFG['mod_dir'] . ' directory from your server</h3><br />';

		$install_mod->install_footer();
	break;

	case 'update':
		echo '<h1>' . $CFG['mod_title'] . ' Update</h1>';
		echo '<p>Table Prefix :: ' . $table_prefix . '<br />';
		echo 'Database Type :: ' . $db->sql_layer . '</p>';
		echo '<p>To check the installation please click <a href="./install_check.php">here</a>.</p>';
		
		if ($CFG['clear_cache_update'])
		{
			echo '<br /><h3>Clearing cache...</h3>';
			$install_mod->clear_cache();
		}

		// Arcade Config
		$sql = 'SELECT *
			FROM ' . ARCADE_CONFIG_TABLE;
		$result = $db->sql_query($sql);

		$arcade_config = array();
		while ($row = $db->sql_fetchrow($result))
		{
			$arcade_config[$row['config_name']] = $row['config_value'];
		}
		$db->sql_freeresult($result);

		if (version_compare($install_mod->format_version($arcade_config['version']), $install_mod->format_version($CFG['mod_version']), '<'))
		{
			switch ($arcade_config['version'])
			{
				case '0.1.0 BETA':
					echo '<br /><h3>Updating database from version 0.1.0 BETA to 0.2.0 BETA...</h3>';
					$phpbb_db_tools->perform_schema_changes($CFG['update_schema_changes']['0.2.0']);
					$install_mod->set_config('download_list', false);

					echo '<br /><h3>Inserting mod modules and clearing module cache...</h3>';
					// Add New ACP Modules...
					// No longer need since we remove all and re-add them in 0.5.0
					// $install_mod->update_modules($CFG['mod_modules'][0]['parent_module_data']['module_langname'], $CFG['mod_modules'][0]['parent_module_data']['module_class'], array($CFG['mod_modules'][0]['module_data'][9]));

					// Add New UCP Module
					$install_mod->create_modules($CFG['mod_modules'][1]['parent_module_data'], $CFG['mod_modules'][1]['module_data']);

				case '0.2.0 BETA':
					echo '<br /><h3>Updating database from version 0.2.0 BETA to 0.3.0 BETA...</h3>';
					$phpbb_db_tools->perform_schema_changes($CFG['update_schema_changes']['0.3.0']);

				case '0.3.0 BETA':
					echo '<br /><h3>Updating database from version 0.3.0 BETA to 0.3.1 BETA...</h3>';
					$phpbb_db_tools->perform_schema_changes($CFG['update_schema_changes']['0.3.1']);
					$install_mod->set_config('copyright', 'Powered by phpBB Arcade &copy; 2008 <a href="http:// www.jeffrusso.net">JRSweets</a>');

				case '0.3.1':
					echo '<br /><h3>Updating database from version 0.3.1 BETA to 0.3.2 BETA...</h3>';
					$install_mod->set_config('protect_amod', true);
					$install_mod->set_config('protect_ibpro', true);
					$install_mod->set_config('protect_v3arcade', true);

				case '0.3.2':
					echo '<br /><h3>Updating database from version 0.3.2 BETA to 0.4.0 BETA...</h3>';
					$phpbb_db_tools->perform_schema_changes($CFG['update_schema_changes']['0.4.0']);

				case '0.4.0':
					echo '<br /><h3>Updating database from version 0.4.0 BETA to 0.4.1 BETA...</h3>';
					$install_mod->set_config('game_scores', 5);
					$install_mod->set_config('parse_bbcode', true);
					$install_mod->set_config('parse_smilies', true);
					$install_mod->set_config('parse_links', true);
					$install_mod->set_config('game_width', 0);
					$install_mod->set_config('game_height', 0);

				case '0.4.1':
					echo '<br /><h3>Updating database from version 0.4.1 BETA to 0.4.2 BETA...</h3>';
					$phpbb_db_tools->perform_schema_changes($CFG['update_schema_changes']['0.4.2']);
					echo '<br /><h3>Creating mod database tables...</h3>';
					$install_mod->load_tables('0.4.2');
					echo '<br /><h3>Inserting mod modules and clearing module cache...</h3>';
					// Add New ACP Modules...
					// No longer need since we remove all and re-add them in 0.5.0
					// $install_mod->update_modules($CFG['mod_modules'][0]['parent_module_data']['module_langname'], $CFG['mod_modules'][0]['parent_module_data']['module_class'], array($CFG['mod_modules'][0]['module_data'][10]));

					$sql = 'SELECT game_name, game_id
						FROM ' . ARCADE_GAMES_TABLE;
					$result = $db->sql_query($sql);

					$row = $db->sql_fetchrowset($result);
					$db->sql_freeresult($result);

					foreach ($row as $item)
					{
						$sql = 'UPDATE ' . ARCADE_GAMES_TABLE . "
							SET game_name_clean = '" . $db->sql_escape(utf8_clean_string($item['game_name'])) . "'
						WHERE game_id = " . (int) $item['game_id'];
						$db->sql_query($sql);
					}

				case '0.4.2':
					echo '<br /><h3>Updating database from version 0.4.2 BETA to 0.4.3 BETA...</h3>';
					$phpbb_db_tools->perform_schema_changes($CFG['update_schema_changes']['0.4.3']);
					$install_mod->set_config('played_games', 1);
					$install_mod->set_config('played_colour', 'cdcdcd');

				case '0.4.3':
					echo '<br /><h3>Updating database from version 0.4.3 BETA to 0.4.4 BETA...</h3>';
					$install_mod->set_config('online_time', 0);
					$install_mod->set_config('announce_game', false);
					$install_mod->set_config('announce_forum', 0);
					$install_mod->set_config('announce_subject', '[GAME RELEASE] [game_name]');
					$install_mod->set_config('announce_message', '[game_image]
[b]Game Name:[/b] [game_name]
[b]Game Description:[/b] [game_desc]

[game_link]
[download_link]
[stats_link]');

				case '0.4.4':
					echo '<br /><h3>Updating database from version 0.4.4 BETA to 0.4.5 BETA...</h3>';
					echo '<br /><h3>Updating mod permissions and clearing permission cache...</h3>';
					// Add permissions
					$install_mod->add_permissions($CFG['update_permission_options']['0.4.5']);

				case '0.4.5':
					echo '<br /><h3>Updating database from version 0.4.5 BETA to 0.5.0 BETA...</h3>';
					$phpbb_db_tools->perform_schema_changes($CFG['update_schema_changes']['0.5.0']);
					echo '<br /><h3>Creating mod database tables...</h3>';
					$install_mod->load_tables('0.5.0');
					echo '<br /><h3>Inserting mod data...</h3>';
					$install_mod->load_data('schemas/0.5.0schema_data.sql');
					$install_mod->set_config('arcade_leaders_header', 3);
					$install_mod->set_config('resolution_select', 1);
					$install_mod->set_config('display_desc', true);
					$install_mod->set_config('arcade_disable', false);
					$install_mod->set_config('arcade_disable_msg', '');
					$install_mod->set_config('cache_time', 4);

					echo '<br /><h3>Updating category table...</h3>';
					// Ok this is tricky I hope it doesn't mess anything up...
					// First set any links (old value was cat type 1) to new value 2
					$sql = 'UPDATE ' . ARCADE_CATS_TABLE . '
						SET cat_type = 2
						WHERE cat_type = 1';
					$db->sql_query($sql);

					// Now change any old arcade cats with games (old value was cat type 0) to new value 1
					// This makes room for categories without games (like the non postable forums)
					$sql = 'UPDATE ' . ARCADE_CATS_TABLE . '
						SET cat_type = 1
						WHERE cat_type = 0';
					$db->sql_query($sql);

					// Now lets deal with passwords for cats....
					$sql = 'SELECT cat_id, cat_password
						FROM ' . ARCADE_CATS_TABLE . "
						WHERE cat_password <> ''";
					$result = $db->sql_query($sql);

					while ($row = $db->sql_fetchrow($result))
					{
						$sql = 'UPDATE ' . ARCADE_CATS_TABLE . '
							SET ' . $db->sql_build_array('UPDATE', array('cat_password' => phpbb_hash($row['cat_password']))) . '
							WHERE cat_id = ' . (int) $row['cat_id'];
						$db->sql_query($sql);
					}
					$db->sql_freeresult($result);

					// Remove old permissions
					echo '<br /><h3>Updating mod permissions and clearing permission cache...</h3>';
					$old_perm = array();
					$old_perm['global'] = array('u_arcade_view', 'u_arcade_play', 'u_arcade_score', 'u_arcade_rate', 'u_arcade_comment', 'u_arcade_download', 'u_arcade_ignore_playcontrol');
					$install_mod->remove_permissions($old_perm);
					$install_mod->add_permissions($CFG['update_permission_options']['0.5.0']);
					$install_mod->clear_permission_cache();

					echo '<br /><h3>Updating mod modules and clearing module cache...</h3>';
					// Remove all modules from the .MODS tab,  we will next try to create a new tab just for the arcade
					// Also remove the ucp modules and re-add with different auth settings
					$install_mod->remove_modules(array('ACP_ARCADE', 'UCP_ARCADE'), $CFG['module_remove']);
					foreach($CFG['mod_modules'] as $modules)
					{
						$install_mod->create_modules($modules['parent_module_data'], $modules['module_data']);
					}
					
				case '0.5.0':
					echo '<br /><h3>Updating database from version 0.5.0 BETA to 0.5.1 BETA...</h3>';
					$install_mod->set_config('welcome_index', 1);
					$install_mod->set_config('search_index', 1);
					$install_mod->set_config('links_index', 1);
					$install_mod->set_config('welcome_cats', 1);
					$install_mod->set_config('search_cats', 1);
					$install_mod->set_config('links_cats', 1);
					$install_mod->set_config('welcome_stats', 1);
					$install_mod->set_config('search_stats', 1);
					$install_mod->set_config('links_stats', 1);
					
				case '0.5.1':
					echo '<br /><h3>Updating database from version 0.5.1 BETA to 0.6.0 BETA...</h3>';
					$install_mod->set_config('most_downloaded', 5);
					$install_mod->set_config('least_downloaded', 5);
					$install_mod->set_config('limit_play_total_posts', 5);
					echo '<br /><h3>Updating mod modules and clearing module cache...</h3>';
					$install_mod->update_modules($CFG['mod_modules'][0]['parent_module_data']['module_langname'], $CFG['mod_modules'][0]['parent_module_data']['module_class'], array($CFG['mod_modules'][0]['module_data'][2]));
					echo '<br /><h3>Updating mod permissions and clearing permission cache...</h3>';					
					$install_mod->add_arcade_permissions($CFG['update_arcade_permission_options']['0.6.0']);
					
					$sql = 'SELECT auth_option_id
						FROM ' . ACL_ARCADE_OPTIONS_TABLE . "
						WHERE auth_option = 'c_popup'";
					$result = $db->sql_query($sql);
					$auth_option_id = $db->sql_fetchfield('auth_option_id');
					$db->sql_freeresult($result);					
					
					$arcade_roles = array('ROLE_ARCADE_FULL', 'ROLE_ARCADE_LIMITED', 'ROLE_ARCADE_PLAYONLY', 'ROLE_ARCADE_STANDARD', 'ROLE_ARCADE_STANDARD_DOWNLOADS');
					foreach ($arcade_roles as $role)
					{
						$sql = 'SELECT role_id
							FROM ' . ACL_ARCADE_ROLES_TABLE . "
							WHERE role_name = '$role'";
						$result = $db->sql_query($sql);
						$role_id = $db->sql_fetchfield('role_id');
						$db->sql_freeresult($result);
						
						$sql_ary = array(
							'role_id'			=> $role_id, 
							'auth_option_id'	=> $auth_option_id, 
							'auth_setting'		=> 1,
						);
						
						$sql = 'INSERT INTO ' . ACL_ARCADE_ROLES_DATA_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary);
						$db->sql_query($sql);						
					}
				
				case '0.6.0':
					echo '<br /><h3>Updating database from version 0.6.0 BETA to 0.6.1 BETA...</h3>';
					$install_mod->set_config('display_viewtopic', 1);
					$install_mod->set_config('display_memberlist', 1);
					$phpbb_db_tools->perform_schema_changes($CFG['update_schema_changes']['0.6.1']);
					
					echo '<br /><h3>Updating game filesizes...</h3>';	
					$sql = 'SELECT game_id, game_swf, game_files
						FROM ' . ARCADE_GAMES_TABLE;
					$result = $db->sql_query($sql);
					while ($row = $db->sql_fetchrow($result))
					{
						$path = remove_extension($row['game_swf']);
						$game_files_array = array();		
						$game_files_array[] = $phpbb_root_path . $arcade_config['game_path'] . $path . '/' . $row['game_swf'];						
						
						$extra_game_files = ($row['game_files'] != '') ? unserialize($row['game_files']) : '';
						if (is_array($extra_game_files))
						{
							foreach ($extra_game_files as $file)
							{
								$file = $phpbb_root_path . $file;
								if (file_exists($file))
								{
									$game_files_array[] = $file;
								}
							}
						}
						
						$filesize = get_filesize($game_files_array);

						if ($filesize)
						{
							$sql = 'UPDATE ' . ARCADE_GAMES_TABLE . '
								SET game_filesize = '. (int) $filesize . '
								WHERE game_id = ' . (int) $row['game_id'];
							$db->sql_query($sql);		
						}					
					}
					$db->sql_freeresult($result);		
				
				break;

				default:
				break;
			}

			// Set arcade version config value to latest version
			$install_mod->set_config('version', $CFG['mod_version']);

			if ($CFG['clear_cache_update'])
			{
				echo '<br /><h3>Clearing cache...</h3>';
				$install_mod->clear_cache();
			}

			echo '<br /><h3> ' . $CFG['mod_title'] . ' has been updated to the latest version</h3><br />';
			echo '<h3>Remove ' . $CFG['mod_dir'] . ' directory from your server</h3><br />';
		}
		else
		{
			echo '<br /><h3> The latest version of ' . $CFG['mod_title'] . ' is already installed.</h3><br />';
			echo '<h3>Remove ' . $CFG['mod_dir'] . ' directory from your server</h3><br />';
		}

		$install_mod->install_footer();
	break;

	default:
		$install_mod->install_form();
		$install_mod->install_footer();
	break;
}

$db->sql_return_on_error(false);

if (function_exists('exit_handler'))
{
	exit_handler();
}

/*
* Mod install class
*/
class install_mod
{
	function install_mod()
	{
		global $phpbb_root_path, $phpEx, $user, $table_prefix;

		require($phpbb_root_path . 'includes/functions_module.' . $phpEx);
		require($phpbb_root_path . 'includes/acp/acp_modules.' . $phpEx);
		require($phpbb_root_path . 'includes/acp/auth.' . $phpEx);
		require($phpbb_root_path . 'includes/functions_install.' . $phpEx);
		require($phpbb_root_path . 'includes/functions_admin.' . $phpEx);

		require($phpbb_root_path . 'includes/arcade/arcade_constants.' . $phpEx);
		require($phpbb_root_path . 'includes/auth_arcade.' . $phpEx);
		require($phpbb_root_path . 'includes/acp/auth_arcade.' . $phpEx);
		require($phpbb_root_path . 'includes/arcade/functions_arcade.' . $phpEx);

		$user->add_lang('acp/modules');

		$module_info = new p_master();
		$module_info->add_mod_info('acp');
		$module_info->add_mod_info('mcp');
		$module_info->add_mod_info('ucp');
	}

	function install_header($title, $dir = '', $lang = '')
	{
		global $phpbb_root_path;

	header('Content-type: text/html; charset=UTF-8');
	echo '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
	<html xmlns="http://www.w3.org/1999/xhtml" dir="' . $dir . '" lang="' . $lang . '" xml:lang="' . $lang . '">
	<head>

	<meta http-equiv="content-type" content="text/html; charset=UTF-8" />
	<meta http-equiv="content-language" content="' . $lang . '" />
	<meta http-equiv="content-style-type" content="text/css" />
	<meta http-equiv="imagetoolbar" content="no" />

	<style type="text/css" media="screen">
		blockquote {
			background: #ebebeb none 6px 8px no-repeat;
			border: 1px solid #dbdbdb;
			font-size: 0.95em;
			margin: 0.5em 1px 0 25px;
			overflow: hidden;
			padding: 5px;
			background-color: #EBEADD;
			border-color:#DBDBCE;
		}
	</style>

	<title>' . $title . '</title>

	<link href="' . $phpbb_root_path . 'adm/style/admin.css" rel="stylesheet" type="text/css" media="screen" />

	<script type="text/javascript">
	<!--
	function togglemenu(id)
	{
		if (document.getElementById(id))
		{
			if (document.getElementById(id).style.display == \'none\')
			{
				document.getElementById(id).style.display = \'inline\';
			}
			else
			{
				document.getElementById(id).style.display = \'none\';
			}
		}
	}
	-->
	</script>

	</head>

	<body>
	<div id="wrap">
		<div id="page-header">
		<h1>Installation Panel</h1>
		</div>

		<div id="page-body">
			<div id="acp">
			<div class="panel">
				<span class="corners-top"><span></span></span>
					<div id="content">
						<div id="main">';

	}

	function install_form()
	{
		global $CFG, $db, $table_prefix;

		$tables = get_tables($db);
		$installed_version = $install_option = $update_option = $uninstall_option = '';
		if (in_array($table_prefix . 'arcade_config', $tables))
		{
			$sql = 'SELECT config_value
				FROM ' . $table_prefix . "arcade_config
				WHERE config_name = 'version'";
			$result = $db->sql_query($sql);
			$installed_version = (string) $db->sql_fetchfield('config_value');
			$db->sql_freeresult($result);
		}
		unset($tables);

		if ($installed_version != '')
		{
			if (version_compare($this->format_version($installed_version), $this->format_version($CFG['mod_version']), '<'))
			{
				$update_option = '<option value="">---- Update Options ----</option>
					<option value="update">Update to latest version of ' . $CFG['mod_title'] . ' (' . $installed_version . ' to ' . $CFG['mod_version'] . ')</option>';
				$uninstall_option = '<option value="">---- Uninstall Options ----</option>
					<option value="uninstall">Uninstall ' . $CFG['mod_title'] . '</option>';
			}
			else
			{
				$uninstall_option = '<option value="">---- Uninstall Options ----</option>
					<option value="uninstall">Uninstall ' . $CFG['mod_title'] . '</option>';
			}
		}
		else
		{
			$install_option = '<option value="">---- Install Options ----</option>
				<option value="install">First Time Install of ' . $CFG['mod_title'] . ' ' . $CFG['mod_version'] . '</option>';
		}

		echo '<h1>' . $CFG['mod_title'] . ' Installation Options</h1>';
		echo '<p>Table Prefix :: ' . $table_prefix . '<br />';
		echo 'Database Type :: ' . $db->sql_layer . '</p>';

		echo '	<form action="' . $_SERVER['PHP_SELF'] . '" method="post">
				<p>This script will install, uninstall or upgrade the tables for the ' . $CFG['mod_title'] . '.
				<br />To check the installation please click <a href="./install_check.php">here</a>.</p>
				<p><b>Backup data tables before going on!</b></p>
				<p><select name="mode">
					' . $install_option . '
					' . $update_option . '
					' . $uninstall_option . '
					</select></p>
				<p><input type="submit" value="Submit" class="button2"></p>
				<p><b>Once you have finished with this script, delete it from your server!</b></p>
			</form>';
	}

	function install_footer()
	{
		global $CFG;

						echo '<p>Click <a href="./../index.php">here</a> to return to the board index.</p></div>
					</div>
				<span class="corners-bottom"><span></span></span>
			</div>
			</div>
		</div>

		<div id="page-footer">
			' . $CFG['mod_copyright'] . '
			Powered by phpBB &copy; 2000, 2002, 2005, 2007 <a href="http:// www.phpbb.com/">phpBB Group</a>

		</div>
	</div>

	</body>
	</html>';

		garbage_collection();
	}

	function clear_cache()
	{
		global $cache;

		$cache->purge();
	}

	function clear_permission_cache()
	{
		global $cache, $table_prefix, $phpbb_root_path, $phpEx;

		$auth_admin = new auth_admin();
		$cache->destroy('_acl_options');
		$auth_admin->acl_clear_prefetch();

		$auth_admin = new auth_arcade_admin();
		$cache->destroy('_acl_arcade_options');
		$auth_admin->acl_clear_prefetch();
	}

	function add_permissions($options)
	{
		global $cache;

		$auth_admin = new auth_admin();
		$auth_admin->acl_add_option($options);
	}

	function add_arcade_permissions($options)
	{
		global $cache, $table_prefix, $phpbb_root_path, $phpEx;

		$auth_admin = new auth_arcade_admin();
		$auth_admin->acl_add_option($options);
	}

	function remove_permissions($options)
	{
		global $db, $cache;

		$auth_option_id = array();
		if (!empty($options['local']))
		{
			foreach($options['local'] as $local)
			{
				$sql = 'SELECT auth_option_id
					FROM ' . ACL_OPTIONS_TABLE . "
				WHERE auth_option = '" . $db->sql_escape($local) . "'";

				$result = $db->sql_query($sql);
				while ($row = $db->sql_fetchrow($result))
				{
					$auth_option_id[] = $row['auth_option_id'];
				}
				$db->sql_freeresult($result);
			}
		}

		if (!empty($options['global']))
		{
			foreach($options['global'] as $global)
			{
				$sql = 'SELECT auth_option_id
					FROM ' . ACL_OPTIONS_TABLE . "
				WHERE auth_option = '" . $db->sql_escape($global) . "'";

				$result = $db->sql_query($sql);
				while ($row = $db->sql_fetchrow($result))
				{
					$auth_option_id[] = $row['auth_option_id'];
				}
				$db->sql_freeresult($result);
			}
		}

		// We now have a list of ids we need to remove from the auth tables...
		if (!empty($auth_option_id))
		{
			$tables = array(ACL_OPTIONS_TABLE, ACL_GROUPS_TABLE, ACL_USERS_TABLE, ACL_ROLES_DATA_TABLE);

			foreach ($tables as $table)
			{
				$sql = "DELETE FROM $table
					WHERE " . $db->sql_in_set('auth_option_id', array_map('intval', $auth_option_id));
				$db->sql_query($sql);
			}

			$auth_admin = new auth_admin();
			$cache->destroy('_acl_options');
			$auth_admin->acl_clear_prefetch();
		}
	}

	function remove_arcade_permissions($options)
	{
		global $db, $cache, $table_prefix, $phpbb_root_path, $phpEx;

		$auth_option_id = array();
		if (!empty($options['local']))
		{
			foreach($options['local'] as $local)
			{
				$sql = 'SELECT auth_option_id
					FROM ' . ACL_ARCADE_OPTIONS_TABLE . "
				WHERE auth_option = '" . $db->sql_escape($local) . "'";

				$result = $db->sql_query($sql);
				while ($row = $db->sql_fetchrow($result))
				{
					$auth_option_id[] = $row['auth_option_id'];
				}
				$db->sql_freeresult($result);
			}
		}

		if (!empty($options['global']))
		{
			foreach($options['global'] as $global)
			{
				$sql = 'SELECT auth_option_id
					FROM ' . ACL_ARCADE_OPTIONS_TABLE . "
				WHERE auth_option = '" . $db->sql_escape($global) . "'";

				$result = $db->sql_query($sql);
				while ($row = $db->sql_fetchrow($result))
				{
					$auth_option_id[] = $row['auth_option_id'];
				}
				$db->sql_freeresult($result);
			}
		}

		// We now have a list of ids we need to remove from the auth tables...
		if (!empty($auth_option_id))
		{
			$tables = array(ACL_ARCADE_OPTIONS_TABLE, ACL_ARCADE_GROUPS_TABLE, ACL_ARCADE_USERS_TABLE, ACL_ARCADE_ROLES_DATA_TABLE);

			foreach ($tables as $table)
			{
				$sql = "DELETE FROM $table
					WHERE " . $db->sql_in_set('auth_option_id', array_map('intval', $auth_option_id));
				$db->sql_query($sql);
			}

			$auth_admin = new auth_arcade_admin();
			$cache->destroy('_acl_arcade_options');
			$auth_admin->acl_clear_prefetch();
		}
	}

	function db_error($error, $sql, $line, $file)
	{
		global $db;

		echo '<p style="color: red;">' . basename($file) . ' [ ' . $line . ' ]</p><p style="color: red;">SQL : <textarea style="font-family:\'Courier New\',monospace;width:99%" rows="5" cols="10">' . preg_replace('/\t(AND|OR)(\W)/', "\$1\$2", htmlspecialchars(preg_replace('/[\s]*[\n\r\t]+[\n\r\s\t]*/', "\n", $sql))) . '</textarea></p><p style="color: red;"><b>' . $error . '</b></p>';
		// Rollback if in transaction
		if ($db->transaction)
		{
			$db->sql_transaction('rollback');
		}
	}

	function create_modules($parent_module_data, $module_data)
	{
		global $phpbb_root_path, $phpEx, $db;
		$_module = &new acp_modules();
		$_module->module_class = $parent_module_data['module_class'];

		$db->sql_error_triggered = false;

		// If the module class is acp we add it to the MODS tab in the ACP
		if ($parent_module_data['module_class'] == 'acp')
		{
			$sql = 'SELECT module_id
				FROM ' . MODULES_TABLE . "
				WHERE module_langname = 'ACP_CAT_ARCADE'";
			$result = $db->sql_query($sql);
			$row = $db->sql_fetchrow($result);
			$db->sql_freeresult($result);

			// Create .MODS tab if missing and get id again
			if(!$row)
			{
				$dot_mods = array(
					'module_basename' 	=> '',
					'module_enabled'	=> '1',
					'module_display' 	=> '1',
					'parent_id' 		=> '0',
					'module_class' 		=> 'acp',
					'module_langname' 	=> 'ACP_CAT_ARCADE',
					'module_mode' 		=> '',
					'module_auth' 		=> '',
				);

				$_module->update_module_data($dot_mods, true);

				$sql = 'SELECT module_id
					FROM ' . MODULES_TABLE . "
					WHERE module_langname = 'ACP_CAT_ARCADE'";
				$result = $db->sql_query($sql);
				$row = $db->sql_fetchrow($result);
				$db->sql_freeresult($result);
			}

			$parent_module_data['parent_id'] = $row['module_id'];
		}

		// Add category
		$_module->update_module_data($parent_module_data, true);
		$_module->remove_cache_file();

		// Check for last sql error happened
		if ($db->sql_error_triggered)
		{
			$db->sql_error_triggered = false;
			$error = $db->sql_error($db->sql_error_sql);
			$this->db_error($error['message'], $db->sql_error_sql, __LINE__, __FILE__);
		}
		else
		{
			add_log('admin', 'LOG_MODULE_ADD', $_module->lang_name($parent_module_data['module_langname']));
		}

		$sql = 'SELECT module_id
			FROM ' . MODULES_TABLE . "
			WHERE module_langname = '{$parent_module_data['module_langname']}'
				AND module_class = '{$parent_module_data['module_class']}'";
		$result = $db->sql_query($sql);
		$row = $db->sql_fetchrow($result);
		$db->sql_freeresult($result);

		for ($i = 0, $count = sizeof($module_data);$i < $count; $i++)
		{
			$module_data[$i]['parent_id'] = $row['module_id'];
			$_module->update_module_data($module_data[$i], true);
			$_module->remove_cache_file();

			// Check for last sql error happened
			if ($db->sql_error_triggered)
			{
				$db->sql_error_triggered = false;
				$error = $db->sql_error($db->sql_error_sql);
				$this->db_error($error['message'], $db->sql_error_sql, __LINE__, __FILE__);
			}
			else
			{
				add_log('admin', 'LOG_MODULE_ADD', $_module->lang_name($module_data[$i]['module_langname']));
			}
		}

		return;
	}

	function update_modules($parent_module_langname, $parent_module_class, $module_data)
	{
		global $phpbb_root_path, $phpEx, $db;
		$_module = &new acp_modules();
		$_module->module_class = $parent_module_class;

		$db->sql_error_triggered = false;

		$sql = 'SELECT module_id
			FROM ' . MODULES_TABLE . "
			WHERE module_langname = '$parent_module_langname'
				AND module_class = '$parent_module_class'";
		$result = $db->sql_query($sql);
		$row = $db->sql_fetchrow($result);
		$db->sql_freeresult($result);

		for ($i = 0, $count = sizeof($module_data);$i < $count; $i++)
		{
			$module_data[$i]['parent_id'] = $row['module_id'];
			$_module->update_module_data($module_data[$i], true);
			$_module->remove_cache_file();

			// Check for last sql error happened
			if ($db->sql_error_triggered)
			{
				$db->sql_error_triggered = false;
				$error = $db->sql_error($db->sql_error_sql);
				$this->db_error($error['message'], $db->sql_error_sql, __LINE__, __FILE__);
			}
			else
			{
				add_log('admin', 'LOG_MODULE_ADD', $_module->lang_name($module_data[$i]['module_langname']));
			}
		}

		return;
	}

	function remove_modules($parent_module_data, $module_data)
	{
		global $db;
		$_module = &new acp_modules();

		$db->sql_error_triggered = false;

		if (!empty($module_data))
		{
			$sql = 'SELECT module_id, module_class
				FROM ' . MODULES_TABLE . '
				WHERE ' . $db->sql_in_set('module_basename', $module_data);
			$result = $db->sql_query($sql);
			while ($row = $db->sql_fetchrow($result))
			{
				$_module->module_class = $row['module_class'];
				$_module->delete_module($row['module_id']);
				// Check for last sql error happened
				if ($db->sql_error_triggered)
				{
					$db->sql_error_triggered = false;
					$error = $db->sql_error($db->sql_error_sql);
					$this->db_error($error['message'], $db->sql_error_sql, __LINE__, __FILE__);
				}
			}
			$db->sql_freeresult($result);
		}

		if (!empty($parent_module_data))
		{
			// Needs to be ordered descending so that we can remove the parent module (tab) last
			$sql = 'SELECT module_id, module_class
				FROM ' . MODULES_TABLE . '
				WHERE ' . $db->sql_in_set('module_langname', $parent_module_data) . '
				ORDER BY module_id DESC';
			$result = $db->sql_query($sql);
			while ($row = $db->sql_fetchrow($result))
			{
				$_module->module_class = $row['module_class'];
				$_module->delete_module($row['module_id']);
				// Check for last sql error happened
				if ($db->sql_error_triggered)
				{
					$db->sql_error_triggered = false;
					$error = $db->sql_error($db->sql_error_sql);
					$this->db_error($error['message'], $db->sql_error_sql, __LINE__, __FILE__);
				}
			}
			$db->sql_freeresult($result);
		}

		return;
	}

	function load_tables($prefix = '')
	{
		global $db, $dbms, $table_prefix;

		$available_dbms = get_available_dbms($dbms);
		// If mysql is chosen, we need to adjust the schema filename slightly to reflect the correct version. ;)
		if ($dbms == 'mysql')
		{
			if (version_compare($db->mysql_version, '4.1.3', '>='))
			{
				$available_dbms[$dbms]['SCHEMA'] .= '_41';
			}
			else
			{
				$available_dbms[$dbms]['SCHEMA'] .= '_40';
			}
		}
		// Ok we have the db info go ahead and read in the relevant schema
		// and work on building the table
		$dbms_schema = 'schemas/' . $prefix . $available_dbms[$dbms]['SCHEMA'] . '_schema.sql';

		// How should we treat this schema?
		$remove_remarks = $available_dbms[$dbms]['COMMENTS'];
		$delimiter = $available_dbms[$dbms]['DELIM'];

		$sql_query = @file_get_contents($dbms_schema);

		$sql_query = preg_replace('#phpbb_#i', $table_prefix, $sql_query);

		$remove_remarks($sql_query);

		$sql_query = split_sql_file($sql_query, $delimiter);

		echo '<blockquote>';
		echo '	<div style="cursor:pointer;cursor:hand;" onclick="if (this.getElementsByTagName(\'div\')[0].style.display != \'block\') { this.getElementsByTagName(\'div\')[0].style.display = \'block\';} else { this.getElementsByTagName(\'div\')[0].style.display = \'none\'; }">';
		echo '		<cite>Results: (click to show/hide)</cite>';
		echo '		<div style="display: none;"><br />';
		foreach ($sql_query as $sql)
		{
			// $sql = trim(str_replace('|', ';', $sql));
			if (!$db->sql_query($sql))
			{
				$error = $db->sql_error();
				$this->db_error($error['message'], $sql, __LINE__, __FILE__);
			}
			else
			{
				echo '<p style="color: green;"><textarea style="font-family:\'Courier New\',monospace;width:99%" rows="5" cols="10">' . preg_replace('/\t(AND|OR)(\W)/', "\$1\$2", htmlspecialchars(preg_replace('/[\s]*[\n\r\t]+[\n\r\s\t]*/', "\n", $sql))) . '</textarea></p>';
			}
		}
		echo '		</div>';
		echo '	</div>';
		echo '</blockquote>';
		unset($sql_query);
	}

	function load_data($file)
	{
		global $db, $dbms, $table_prefix;

		// Ok tables have been built, let's fill in the basic information
		$sql_query = file_get_contents($file);

		// Deal with any special comments
		switch ($dbms)
		{
			case 'mssql':
			case 'mssql_odbc':
				$sql_query = preg_replace('#\# MSSQL IDENTITY (phpbb_[a-z_]+) (ON|OFF) \##s', 'SET IDENTITY_INSERT \1 \2;', $sql_query);
			break;

			case 'postgres':
				$sql_query = preg_replace('#\# POSTGRES (BEGIN|COMMIT) \##s', '\1; ', $sql_query);
			break;
		}

		// Change prefix
		$sql_query = preg_replace('#phpbb_#i', $table_prefix, $sql_query);

		// Change language strings...
		$sql_query = preg_replace_callback('#\{L_([A-Z0-9\-_]*)\}#s', 'adjust_language_keys_callback', $sql_query);

		// Since there is only one schema file we know the comment style and are able to remove it directly with remove_remarks
		remove_remarks($sql_query);
		$sql_query = split_sql_file($sql_query, ';');

		echo '<blockquote>';
		echo '	<div style="cursor:pointer;cursor:hand;" onclick="if (this.getElementsByTagName(\'div\')[0].style.display != \'block\') { this.getElementsByTagName(\'div\')[0].style.display = \'block\';} else { this.getElementsByTagName(\'div\')[0].style.display = \'none\'; }">';
		echo '		<cite>Results: (click to show/hide)</cite>';
		echo '		<div style="display: none;"><br />';
		foreach ($sql_query as $sql)
		{
			// $sql = trim(str_replace('|', ';', $sql));
			if (!$db->sql_query($sql))
			{
				$error = $db->sql_error();
				$this->db_error($error['message'], $sql, __LINE__, __FILE__);
			}
			else
			{
				echo '<p style="color: green;"><textarea style="font-family:\'Courier New\',monospace;width:99%" rows="5" cols="10">' . preg_replace('/\t(AND|OR)(\W)/', "\$1\$2", htmlspecialchars(preg_replace('/[\s]*[\n\r\t]+[\n\r\s\t]*/', "\n", $sql))) . '</textarea></p>';
			}
		}
		echo '		</div>';
		echo '	</div>';
		echo '</blockquote>';
		unset($sql_query);
	}

	function format_version($version)
	{
		return str_replace(' ', '.', $version);
	}

	/**
	* Set config value. Creates missing config entry.
	*/
	function set_config($config_name, $config_value)
	{
		global $db, $cache;

		$sql = 'UPDATE ' . ARCADE_CONFIG_TABLE . "
			SET config_value = '" . $db->sql_escape($config_value) . "'
			WHERE config_name = '" . $db->sql_escape($config_name) . "'";
		$db->sql_query($sql);

		if (!$db->sql_affectedrows() && !isset($this->config[$config_name]))
		{
			$sql = 'INSERT INTO ' . ARCADE_CONFIG_TABLE . ' ' . $db->sql_build_array('INSERT', array(
				'config_name'	=> $config_name,
				'config_value'	=> $config_value));
			$db->sql_query($sql);
		}

		$this->config[$config_name] = $config_value;

		// Destroy the cache of the config
		// because the values have changed
		$cache->destroy('_arcade');
	}
}

?>