/*

 $Id: $

*/

/*
  This first section is optional, however its probably the best method
  of running phpBB on Oracle. If you already have a tablespace and user created
  for phpBB you can leave this section commented out!

  The first set of statements create a phpBB tablespace and a phpBB user,
  make sure you change the password of the phpBB user before you run this script!!
*/

/*
CREATE TABLESPACE "PHPBB"
	LOGGING 
	DATAFILE 'E:\ORACLE\ORADATA\LOCAL\PHPBB.ora' 
	SIZE 10M
	AUTOEXTEND ON NEXT 10M
	MAXSIZE 100M;

CREATE USER "PHPBB" 
	PROFILE "DEFAULT" 
	IDENTIFIED BY "phpbb_password" 
	DEFAULT TABLESPACE "PHPBB" 
	QUOTA UNLIMITED ON "PHPBB" 
	ACCOUNT UNLOCK;

GRANT ANALYZE ANY TO "PHPBB";
GRANT CREATE SEQUENCE TO "PHPBB";
GRANT CREATE SESSION TO "PHPBB";
GRANT CREATE TABLE TO "PHPBB";
GRANT CREATE TRIGGER TO "PHPBB";
GRANT CREATE VIEW TO "PHPBB";
GRANT "CONNECT" TO "PHPBB";

COMMIT;
DISCONNECT;

CONNECT phpbb/phpbb_password;
*/
/*
	Table: 'phpbb_acl_arcade_groups'
*/
CREATE TABLE phpbb_acl_arcade_groups (
	group_id number(8) DEFAULT '0' NOT NULL,
	cat_id number(8) DEFAULT '0' NOT NULL,
	auth_option_id number(8) DEFAULT '0' NOT NULL,
	auth_role_id number(8) DEFAULT '0' NOT NULL,
	auth_setting number(2) DEFAULT '0' NOT NULL
)
/

CREATE INDEX phpbb_acl_arcade_groups_group_id ON phpbb_acl_arcade_groups (group_id)
/
CREATE INDEX phpbb_acl_arcade_groups_auth_opt_id ON phpbb_acl_arcade_groups (auth_option_id)
/
CREATE INDEX phpbb_acl_arcade_groups_auth_role_id ON phpbb_acl_arcade_groups (auth_role_id)
/

/*
	Table: 'phpbb_acl_arcade_options'
*/
CREATE TABLE phpbb_acl_arcade_options (
	auth_option_id number(8) NOT NULL,
	auth_option varchar2(50) DEFAULT '' ,
	is_global number(1) DEFAULT '0' NOT NULL,
	is_local number(1) DEFAULT '0' NOT NULL,
	founder_only number(1) DEFAULT '0' NOT NULL,
	CONSTRAINT pk_phpbb_acl_arcade_options PRIMARY KEY (auth_option_id)
)
/

CREATE INDEX phpbb_acl_arcade_options_auth_option ON phpbb_acl_arcade_options (auth_option)
/

CREATE SEQUENCE phpbb_acl_arcade_options_seq
/

CREATE OR REPLACE TRIGGER t_phpbb_acl_arcade_options
BEFORE INSERT ON phpbb_acl_arcade_options
FOR EACH ROW WHEN (
	new.auth_option_id IS NULL OR new.auth_option_id = 0
)
BEGIN
	SELECT phpbb_acl_arcade_options_seq.nextval
	INTO :new.auth_option_id
	FROM dual;
END;
/


/*
	Table: 'phpbb_acl_arcade_roles'
*/
CREATE TABLE phpbb_acl_arcade_roles (
	role_id number(8) NOT NULL,
	role_name varchar2(765) DEFAULT '' ,
	role_description clob DEFAULT '' ,
	role_type varchar2(10) DEFAULT '' ,
	role_order number(4) DEFAULT '0' NOT NULL,
	CONSTRAINT pk_phpbb_acl_arcade_roles PRIMARY KEY (role_id)
)
/

CREATE INDEX phpbb_acl_arcade_roles_role_type ON phpbb_acl_arcade_roles (role_type)
/
CREATE INDEX phpbb_acl_arcade_roles_role_order ON phpbb_acl_arcade_roles (role_order)
/

CREATE SEQUENCE phpbb_acl_arcade_roles_seq
/

CREATE OR REPLACE TRIGGER t_phpbb_acl_arcade_roles
BEFORE INSERT ON phpbb_acl_arcade_roles
FOR EACH ROW WHEN (
	new.role_id IS NULL OR new.role_id = 0
)
BEGIN
	SELECT phpbb_acl_arcade_roles_seq.nextval
	INTO :new.role_id
	FROM dual;
END;
/


/*
	Table: 'phpbb_acl_arcade_roles_data'
*/
CREATE TABLE phpbb_acl_arcade_roles_data (
	role_id number(8) DEFAULT '0' NOT NULL,
	auth_option_id number(8) DEFAULT '0' NOT NULL,
	auth_setting number(2) DEFAULT '0' NOT NULL,
	CONSTRAINT pk_phpbb_acl_arcade_roles_data PRIMARY KEY (role_id, auth_option_id)
)
/

CREATE INDEX phpbb_acl_arcade_roles_data_ath_op_id ON phpbb_acl_arcade_roles_data (auth_option_id)
/

/*
	Table: 'phpbb_acl_arcade_users'
*/
CREATE TABLE phpbb_acl_arcade_users (
	user_id number(8) DEFAULT '0' NOT NULL,
	cat_id number(8) DEFAULT '0' NOT NULL,
	auth_option_id number(8) DEFAULT '0' NOT NULL,
	auth_role_id number(8) DEFAULT '0' NOT NULL,
	auth_setting number(2) DEFAULT '0' NOT NULL
)
/

CREATE INDEX phpbb_acl_arcade_users_user_id ON phpbb_acl_arcade_users (user_id)
/
CREATE INDEX phpbb_acl_arcade_users_auth_option_id ON phpbb_acl_arcade_users (auth_option_id)
/
CREATE INDEX phpbb_acl_arcade_users_auth_role_id ON phpbb_acl_arcade_users (auth_role_id)
/

