/*

 $Id: $

*/

BEGIN;

/*
	Domain definition
*/

/*
CREATE DOMAIN varchar_ci AS varchar(255) NOT NULL DEFAULT ''::character varying;
*/

/*
	Operation Functions
*/

/*
CREATE FUNCTION _varchar_ci_equal(varchar_ci, varchar_ci) RETURNS boolean AS 'SELECT LOWER($1) = LOWER($2)' LANGUAGE SQL STRICT;
CREATE FUNCTION _varchar_ci_not_equal(varchar_ci, varchar_ci) RETURNS boolean AS 'SELECT LOWER($1) != LOWER($2)' LANGUAGE SQL STRICT;
CREATE FUNCTION _varchar_ci_less_than(varchar_ci, varchar_ci) RETURNS boolean AS 'SELECT LOWER($1) < LOWER($2)' LANGUAGE SQL STRICT;
CREATE FUNCTION _varchar_ci_less_equal(varchar_ci, varchar_ci) RETURNS boolean AS 'SELECT LOWER($1) <= LOWER($2)' LANGUAGE SQL STRICT;
CREATE FUNCTION _varchar_ci_greater_than(varchar_ci, varchar_ci) RETURNS boolean AS 'SELECT LOWER($1) > LOWER($2)' LANGUAGE SQL STRICT;
CREATE FUNCTION _varchar_ci_greater_equals(varchar_ci, varchar_ci) RETURNS boolean AS 'SELECT LOWER($1) >= LOWER($2)' LANGUAGE SQL STRICT;
*/

/*
	Operators
*/
/*
CREATE OPERATOR <(
  PROCEDURE = _varchar_ci_less_than,
  LEFTARG = varchar_ci,
  RIGHTARG = varchar_ci,
  COMMUTATOR = >,
  NEGATOR = >=,
  RESTRICT = scalarltsel,
  JOIN = scalarltjoinsel);

CREATE OPERATOR <=(
  PROCEDURE = _varchar_ci_less_equal,
  LEFTARG = varchar_ci,
  RIGHTARG = varchar_ci,
  COMMUTATOR = >=,
  NEGATOR = >,
  RESTRICT = scalarltsel,
  JOIN = scalarltjoinsel);

CREATE OPERATOR >(
  PROCEDURE = _varchar_ci_greater_than,
  LEFTARG = varchar_ci,
  RIGHTARG = varchar_ci,
  COMMUTATOR = <,
  NEGATOR = <=,
  RESTRICT = scalargtsel,
  JOIN = scalargtjoinsel);

CREATE OPERATOR >=(
  PROCEDURE = _varchar_ci_greater_equals,
  LEFTARG = varchar_ci,
  RIGHTARG = varchar_ci,
  COMMUTATOR = <=,
  NEGATOR = <,
  RESTRICT = scalargtsel,
  JOIN = scalargtjoinsel);

CREATE OPERATOR <>(
  PROCEDURE = _varchar_ci_not_equal,
  LEFTARG = varchar_ci,
  RIGHTARG = varchar_ci,
  COMMUTATOR = <>,
  NEGATOR = =,
  RESTRICT = neqsel,
  JOIN = neqjoinsel);

CREATE OPERATOR =(
  PROCEDURE = _varchar_ci_equal,
  LEFTARG = varchar_ci,
  RIGHTARG = varchar_ci,
  COMMUTATOR = =,
  NEGATOR = <>,
  RESTRICT = eqsel,
  JOIN = eqjoinsel,
  HASHES,
  MERGES,
  SORT1= <);
*/
/*
	Table: 'phpbb_acl_arcade_groups'
*/
CREATE TABLE phpbb_acl_arcade_groups (
	group_id INT4 DEFAULT '0' NOT NULL CHECK (group_id >= 0),
	cat_id INT4 DEFAULT '0' NOT NULL CHECK (cat_id >= 0),
	auth_option_id INT4 DEFAULT '0' NOT NULL CHECK (auth_option_id >= 0),
	auth_role_id INT4 DEFAULT '0' NOT NULL CHECK (auth_role_id >= 0),
	auth_setting INT2 DEFAULT '0' NOT NULL
);

CREATE INDEX phpbb_acl_arcade_groups_group_id ON phpbb_acl_arcade_groups (group_id);
CREATE INDEX phpbb_acl_arcade_groups_auth_opt_id ON phpbb_acl_arcade_groups (auth_option_id);
CREATE INDEX phpbb_acl_arcade_groups_auth_role_id ON phpbb_acl_arcade_groups (auth_role_id);

/*
	Table: 'phpbb_acl_arcade_options'
*/
CREATE SEQUENCE phpbb_acl_arcade_options_seq;

CREATE TABLE phpbb_acl_arcade_options (
	auth_option_id INT4 DEFAULT nextval('phpbb_acl_arcade_options_seq'),
	auth_option varchar(50) DEFAULT '' NOT NULL,
	is_global INT2 DEFAULT '0' NOT NULL CHECK (is_global >= 0),
	is_local INT2 DEFAULT '0' NOT NULL CHECK (is_local >= 0),
	founder_only INT2 DEFAULT '0' NOT NULL CHECK (founder_only >= 0),
	PRIMARY KEY (auth_option_id)
);

CREATE INDEX phpbb_acl_arcade_options_auth_option ON phpbb_acl_arcade_options (auth_option);

/*
	Table: 'phpbb_acl_arcade_roles'
*/
CREATE SEQUENCE phpbb_acl_arcade_roles_seq;

CREATE TABLE phpbb_acl_arcade_roles (
	role_id INT4 DEFAULT nextval('phpbb_acl_arcade_roles_seq'),
	role_name varchar(255) DEFAULT '' NOT NULL,
	role_description varchar(4000) DEFAULT '' NOT NULL,
	role_type varchar(10) DEFAULT '' NOT NULL,
	role_order INT2 DEFAULT '0' NOT NULL CHECK (role_order >= 0),
	PRIMARY KEY (role_id)
);

CREATE INDEX phpbb_acl_arcade_roles_role_type ON phpbb_acl_arcade_roles (role_type);
CREATE INDEX phpbb_acl_arcade_roles_role_order ON phpbb_acl_arcade_roles (role_order);

/*
	Table: 'phpbb_acl_arcade_roles_data'
*/
CREATE TABLE phpbb_acl_arcade_roles_data (
	role_id INT4 DEFAULT '0' NOT NULL CHECK (role_id >= 0),
	auth_option_id INT4 DEFAULT '0' NOT NULL CHECK (auth_option_id >= 0),
	auth_setting INT2 DEFAULT '0' NOT NULL,
	PRIMARY KEY (role_id, auth_option_id)
);

CREATE INDEX phpbb_acl_arcade_roles_data_ath_op_id ON phpbb_acl_arcade_roles_data (auth_option_id);

/*
	Table: 'phpbb_acl_arcade_users'
*/
CREATE TABLE phpbb_acl_arcade_users (
	user_id INT4 DEFAULT '0' NOT NULL CHECK (user_id >= 0),
	cat_id INT4 DEFAULT '0' NOT NULL CHECK (cat_id >= 0),
	auth_option_id INT4 DEFAULT '0' NOT NULL CHECK (auth_option_id >= 0),
	auth_role_id INT4 DEFAULT '0' NOT NULL CHECK (auth_role_id >= 0),
	auth_setting INT2 DEFAULT '0' NOT NULL
);

CREATE INDEX phpbb_acl_arcade_users_user_id ON phpbb_acl_arcade_users (user_id);
CREATE INDEX phpbb_acl_arcade_users_auth_option_id ON phpbb_acl_arcade_users (auth_option_id);
CREATE INDEX phpbb_acl_arcade_users_auth_role_id ON phpbb_acl_arcade_users (auth_role_id);


COMMIT;