#
# $Id: $
#


# Table: 'phpbb_acl_arcade_groups'
CREATE TABLE phpbb_acl_arcade_groups (
	group_id INTEGER DEFAULT 0 NOT NULL,
	cat_id INTEGER DEFAULT 0 NOT NULL,
	auth_option_id INTEGER DEFAULT 0 NOT NULL,
	auth_role_id INTEGER DEFAULT 0 NOT NULL,
	auth_setting INTEGER DEFAULT 0 NOT NULL
);;

CREATE INDEX phpbb_acl_arcade_groups_group_id ON phpbb_acl_arcade_groups(group_id);;
CREATE INDEX phpbb_acl_arcade_groups_auth_opt_id ON phpbb_acl_arcade_groups(auth_option_id);;
CREATE INDEX phpbb_acl_arcade_groups_auth_role_id ON phpbb_acl_arcade_groups(auth_role_id);;

# Table: 'phpbb_acl_arcade_options'
CREATE TABLE phpbb_acl_arcade_options (
	auth_option_id INTEGER NOT NULL,
	auth_option VARCHAR(50) CHARACTER SET NONE DEFAULT '' NOT NULL,
	is_global INTEGER DEFAULT 0 NOT NULL,
	is_local INTEGER DEFAULT 0 NOT NULL,
	founder_only INTEGER DEFAULT 0 NOT NULL
);;

ALTER TABLE phpbb_acl_arcade_options ADD PRIMARY KEY (auth_option_id);;

CREATE INDEX phpbb_acl_arcade_options_auth_option ON phpbb_acl_arcade_options(auth_option);;

CREATE GENERATOR phpbb_acl_arcade_options_gen;;
SET GENERATOR phpbb_acl_arcade_options_gen TO 0;;

CREATE TRIGGER t_phpbb_acl_arcade_options FOR phpbb_acl_arcade_options
BEFORE INSERT
AS
BEGIN
	NEW.auth_option_id = GEN_ID(phpbb_acl_arcade_options_gen, 1);
END;;


# Table: 'phpbb_acl_arcade_roles'
CREATE TABLE phpbb_acl_arcade_roles (
	role_id INTEGER NOT NULL,
	role_name VARCHAR(255) CHARACTER SET UTF8 DEFAULT '' NOT NULL COLLATE UNICODE,
	role_description BLOB SUB_TYPE TEXT CHARACTER SET UTF8 DEFAULT '' NOT NULL,
	role_type VARCHAR(10) CHARACTER SET NONE DEFAULT '' NOT NULL,
	role_order INTEGER DEFAULT 0 NOT NULL
);;

ALTER TABLE phpbb_acl_arcade_roles ADD PRIMARY KEY (role_id);;

CREATE INDEX phpbb_acl_arcade_roles_role_type ON phpbb_acl_arcade_roles(role_type);;
CREATE INDEX phpbb_acl_arcade_roles_role_order ON phpbb_acl_arcade_roles(role_order);;

CREATE GENERATOR phpbb_acl_arcade_roles_gen;;
SET GENERATOR phpbb_acl_arcade_roles_gen TO 0;;

CREATE TRIGGER t_phpbb_acl_arcade_roles FOR phpbb_acl_arcade_roles
BEFORE INSERT
AS
BEGIN
	NEW.role_id = GEN_ID(phpbb_acl_arcade_roles_gen, 1);
END;;


# Table: 'phpbb_acl_arcade_roles_data'
CREATE TABLE phpbb_acl_arcade_roles_data (
	role_id INTEGER DEFAULT 0 NOT NULL,
	auth_option_id INTEGER DEFAULT 0 NOT NULL,
	auth_setting INTEGER DEFAULT 0 NOT NULL
);;

ALTER TABLE phpbb_acl_arcade_roles_data ADD PRIMARY KEY (role_id, auth_option_id);;

CREATE INDEX phpbb_acl_arcade_roles_data_ath_op_id ON phpbb_acl_arcade_roles_data(auth_option_id);;

# Table: 'phpbb_acl_arcade_users'
CREATE TABLE phpbb_acl_arcade_users (
	user_id INTEGER DEFAULT 0 NOT NULL,
	cat_id INTEGER DEFAULT 0 NOT NULL,
	auth_option_id INTEGER DEFAULT 0 NOT NULL,
	auth_role_id INTEGER DEFAULT 0 NOT NULL,
	auth_setting INTEGER DEFAULT 0 NOT NULL
);;

CREATE INDEX phpbb_acl_arcade_users_user_id ON phpbb_acl_arcade_users(user_id);;
CREATE INDEX phpbb_acl_arcade_users_auth_option_id ON phpbb_acl_arcade_users(auth_option_id);;
CREATE INDEX phpbb_acl_arcade_users_auth_role_id ON phpbb_acl_arcade_users(auth_role_id);;

# Table: 'phpbb_arcade_access'
CREATE TABLE phpbb_arcade_access (
	cat_id INTEGER DEFAULT 0 NOT NULL,
	user_id INTEGER DEFAULT 0 NOT NULL,
	session_id CHAR(32) CHARACTER SET NONE DEFAULT '' NOT NULL
);;

ALTER TABLE phpbb_arcade_access ADD PRIMARY KEY (cat_id, user_id, session_id);;


# Table: 'phpbb_arcade_categories'
CREATE TABLE phpbb_arcade_categories (
	cat_id INTEGER NOT NULL,
	parent_id INTEGER DEFAULT 0 NOT NULL,
	left_id INTEGER DEFAULT 0 NOT NULL,
	right_id INTEGER DEFAULT 0 NOT NULL,
	cat_parents BLOB SUB_TYPE TEXT CHARACTER SET NONE DEFAULT '' NOT NULL,
	cat_name VARCHAR(255) CHARACTER SET UTF8 DEFAULT '' NOT NULL COLLATE UNICODE,
	cat_desc BLOB SUB_TYPE TEXT CHARACTER SET UTF8 DEFAULT '' NOT NULL,
	cat_desc_bitfield VARCHAR(255) CHARACTER SET NONE DEFAULT '' NOT NULL,
	cat_desc_options INTEGER DEFAULT 7 NOT NULL,
	cat_desc_uid VARCHAR(8) CHARACTER SET NONE DEFAULT '' NOT NULL,
	cat_link VARCHAR(255) CHARACTER SET UTF8 DEFAULT '' NOT NULL COLLATE UNICODE,
	cat_password VARCHAR(40) CHARACTER SET UTF8 DEFAULT '' NOT NULL COLLATE UNICODE,
	cat_style INTEGER DEFAULT 0 NOT NULL,
	cat_display INTEGER DEFAULT 0 NOT NULL,
	cat_image VARCHAR(255) CHARACTER SET NONE DEFAULT '' NOT NULL,
	cat_rules BLOB SUB_TYPE TEXT CHARACTER SET UTF8 DEFAULT '' NOT NULL,
	cat_rules_link VARCHAR(255) CHARACTER SET UTF8 DEFAULT '' NOT NULL COLLATE UNICODE,
	cat_rules_bitfield VARCHAR(255) CHARACTER SET NONE DEFAULT '' NOT NULL,
	cat_rules_options INTEGER DEFAULT 7 NOT NULL,
	cat_rules_uid VARCHAR(8) CHARACTER SET NONE DEFAULT '' NOT NULL,
	cat_games_per_page INTEGER DEFAULT 0 NOT NULL,
	cat_type INTEGER DEFAULT 0 NOT NULL,
	cat_status INTEGER DEFAULT 0 NOT NULL,
	cat_games INTEGER DEFAULT 0 NOT NULL,
	cat_plays INTEGER DEFAULT 0 NOT NULL,
	cat_download INTEGER DEFAULT 1 NOT NULL,
	cat_last_play_game_id INTEGER DEFAULT 0 NOT NULL,
	cat_last_play_game_name VARCHAR(255) CHARACTER SET NONE DEFAULT 0 NOT NULL,
	cat_last_play_user_id INTEGER DEFAULT 0 NOT NULL,
	cat_last_play_score DOUBLE PRECISION DEFAULT 0 NOT NULL,
	cat_last_play_time INTEGER DEFAULT 0 NOT NULL,
	cat_last_game_installdate INTEGER DEFAULT 0 NOT NULL,
	cat_last_play_username VARCHAR(255) CHARACTER SET UTF8 DEFAULT '' NOT NULL COLLATE UNICODE,
	cat_last_play_user_colour VARCHAR(6) CHARACTER SET NONE DEFAULT '' NOT NULL,
	cat_flags INTEGER DEFAULT 32 NOT NULL,
	display_subcat_list INTEGER DEFAULT 1 NOT NULL,
	display_on_index INTEGER DEFAULT 1 NOT NULL
);;

ALTER TABLE phpbb_arcade_categories ADD PRIMARY KEY (cat_id);;

CREATE INDEX phpbb_arcade_categories_left_right_id ON phpbb_arcade_categories(left_id, right_id);;
CREATE INDEX phpbb_arcade_categories_cat_lastplay_game_id ON phpbb_arcade_categories(cat_last_play_game_id);;

CREATE GENERATOR phpbb_arcade_categories_gen;;
SET GENERATOR phpbb_arcade_categories_gen TO 0;;

CREATE TRIGGER t_phpbb_arcade_categories FOR phpbb_arcade_categories
BEFORE INSERT
AS
BEGIN
	NEW.cat_id = GEN_ID(phpbb_arcade_categories_gen, 1);
END;;


# Table: 'phpbb_arcade_config'
CREATE TABLE phpbb_arcade_config (
	config_name VARCHAR(255) CHARACTER SET NONE DEFAULT '' NOT NULL,
	config_value VARCHAR(255) CHARACTER SET UTF8 DEFAULT '' NOT NULL COLLATE UNICODE
);;

ALTER TABLE phpbb_arcade_config ADD PRIMARY KEY (config_name);;


# Table: 'phpbb_arcade_errors'
CREATE TABLE phpbb_arcade_errors (
	error_id INTEGER NOT NULL,
	user_id INTEGER DEFAULT 0 NOT NULL,
	game_id INTEGER DEFAULT 0 NOT NULL,
	score DOUBLE PRECISION DEFAULT 0 NOT NULL,
	error_date INTEGER DEFAULT 0 NOT NULL,
	error_type INTEGER DEFAULT 0 NOT NULL,
	game_type INTEGER DEFAULT 0 NOT NULL,
	submitted_game_type INTEGER DEFAULT 0 NOT NULL,
	played_time INTEGER DEFAULT 0 NOT NULL,
	error_ip VARCHAR(40) CHARACTER SET NONE DEFAULT '' NOT NULL
);;

ALTER TABLE phpbb_arcade_errors ADD PRIMARY KEY (error_id);;


CREATE GENERATOR phpbb_arcade_errors_gen;;
SET GENERATOR phpbb_arcade_errors_gen TO 0;;

CREATE TRIGGER t_phpbb_arcade_errors FOR phpbb_arcade_errors
BEFORE INSERT
AS
BEGIN
	NEW.error_id = GEN_ID(phpbb_arcade_errors_gen, 1);
END;;


# Table: 'phpbb_arcade_favorites'
CREATE TABLE phpbb_arcade_favorites (
	user_id INTEGER DEFAULT 0 NOT NULL,
	game_id INTEGER DEFAULT 0 NOT NULL
);;

ALTER TABLE phpbb_arcade_favorites ADD PRIMARY KEY (user_id, game_id);;


# Table: 'phpbb_arcade_games'
CREATE TABLE phpbb_arcade_games (
	game_id INTEGER NOT NULL,
	game_image VARCHAR(255) CHARACTER SET NONE DEFAULT '' NOT NULL,
	game_desc BLOB SUB_TYPE TEXT CHARACTER SET NONE DEFAULT '' NOT NULL,
	game_highscore DOUBLE PRECISION DEFAULT 0 NOT NULL,
	game_highdate INTEGER DEFAULT 0 NOT NULL,
	game_highuser INTEGER DEFAULT 0 NOT NULL,
	game_files BLOB SUB_TYPE TEXT CHARACTER SET NONE DEFAULT '' NOT NULL,
	game_name VARCHAR(255) CHARACTER SET NONE DEFAULT '' NOT NULL,
	game_name_clean VARCHAR(255) CHARACTER SET NONE DEFAULT '' NOT NULL,
	game_swf VARCHAR(255) CHARACTER SET NONE DEFAULT '' NOT NULL,
	game_scorevar VARCHAR(255) CHARACTER SET NONE DEFAULT '' NOT NULL,
	game_type INTEGER DEFAULT 0 NOT NULL,
	game_width INTEGER DEFAULT 550 NOT NULL,
	game_height INTEGER DEFAULT 380 NOT NULL,
	game_installdate INTEGER DEFAULT 0 NOT NULL,
	game_filesize INTEGER DEFAULT 0 NOT NULL,
	game_scoretype INTEGER DEFAULT 0 NOT NULL,
	game_order INTEGER DEFAULT 0 NOT NULL,
	game_plays INTEGER DEFAULT 0 NOT NULL,
	game_votetotal INTEGER DEFAULT 0 NOT NULL,
	game_votesum INTEGER DEFAULT 0 NOT NULL,
	game_download_total INTEGER DEFAULT 0 NOT NULL,
	game_download INTEGER DEFAULT 1 NOT NULL,
	cat_id INTEGER DEFAULT 0 NOT NULL
);;

ALTER TABLE phpbb_arcade_games ADD PRIMARY KEY (game_id);;


CREATE GENERATOR phpbb_arcade_games_gen;;
SET GENERATOR phpbb_arcade_games_gen TO 0;;

CREATE TRIGGER t_phpbb_arcade_games FOR phpbb_arcade_games
BEFORE INSERT
AS
BEGIN
	NEW.game_id = GEN_ID(phpbb_arcade_games_gen, 1);
END;;


# Table: 'phpbb_arcade_game_rating'
CREATE TABLE phpbb_arcade_game_rating (
	game_id INTEGER DEFAULT 0 NOT NULL,
	user_id INTEGER DEFAULT 0 NOT NULL,
	game_rating INTEGER DEFAULT 0 NOT NULL,
	rating_date INTEGER DEFAULT 0 NOT NULL,
	user_ip VARCHAR(40) CHARACTER SET NONE DEFAULT '' NOT NULL
);;

ALTER TABLE phpbb_arcade_game_rating ADD PRIMARY KEY (game_id, user_id);;


# Table: 'phpbb_arcade_scores'
CREATE TABLE phpbb_arcade_scores (
	game_id INTEGER DEFAULT 0 NOT NULL,
	user_id INTEGER DEFAULT 0 NOT NULL,
	score DOUBLE PRECISION DEFAULT 0 NOT NULL,
	comment_text BLOB SUB_TYPE TEXT CHARACTER SET NONE DEFAULT '' NOT NULL,
	comment_bitfield VARCHAR(255) CHARACTER SET NONE DEFAULT '' NOT NULL,
	comment_options INTEGER DEFAULT 7 NOT NULL,
	comment_uid VARCHAR(8) CHARACTER SET NONE DEFAULT '' NOT NULL,
	score_date INTEGER DEFAULT 0 NOT NULL,
	total_time INTEGER DEFAULT 0 NOT NULL,
	total_plays INTEGER DEFAULT 0 NOT NULL
);;

ALTER TABLE phpbb_arcade_scores ADD PRIMARY KEY (game_id, user_id);;


# Table: 'phpbb_arcade_sessions'
CREATE TABLE phpbb_arcade_sessions (
	session_id CHAR(32) CHARACTER SET NONE DEFAULT '' NOT NULL,
	game_id INTEGER DEFAULT 0 NOT NULL,
	user_id INTEGER DEFAULT 0 NOT NULL,
	game_type INTEGER DEFAULT 0 NOT NULL,
	randchar1 VARCHAR(30) CHARACTER SET NONE DEFAULT '' NOT NULL,
	randchar2 VARCHAR(30) CHARACTER SET NONE DEFAULT '' NOT NULL,
	randgid1 VARCHAR(30) CHARACTER SET NONE DEFAULT '' NOT NULL,
	randgid2 VARCHAR(30) CHARACTER SET NONE DEFAULT '' NOT NULL,
	start_time INTEGER DEFAULT 0 NOT NULL
);;

ALTER TABLE phpbb_arcade_sessions ADD PRIMARY KEY (session_id);;


# Table: 'phpbb_arcade_download'
CREATE TABLE phpbb_arcade_download (
	user_id INTEGER DEFAULT 0 NOT NULL,
	game_id INTEGER DEFAULT 0 NOT NULL,
	total INTEGER DEFAULT 0 NOT NULL,
	download_time INTEGER DEFAULT 0 NOT NULL
);;

ALTER TABLE phpbb_arcade_download ADD PRIMARY KEY (user_id, game_id);;


