/*

 $Id: $

*/

BEGIN TRANSACTION
GO

/*
	Table: 'phpbb_acl_arcade_groups'
*/
CREATE TABLE [phpbb_acl_arcade_groups] (
	[group_id] [int] DEFAULT (0) NOT NULL ,
	[cat_id] [int] DEFAULT (0) NOT NULL ,
	[auth_option_id] [int] DEFAULT (0) NOT NULL ,
	[auth_role_id] [int] DEFAULT (0) NOT NULL ,
	[auth_setting] [int] DEFAULT (0) NOT NULL 
) ON [PRIMARY]
GO

CREATE  INDEX [group_id] ON [phpbb_acl_arcade_groups]([group_id]) ON [PRIMARY]
GO

CREATE  INDEX [auth_opt_id] ON [phpbb_acl_arcade_groups]([auth_option_id]) ON [PRIMARY]
GO

CREATE  INDEX [auth_role_id] ON [phpbb_acl_arcade_groups]([auth_role_id]) ON [PRIMARY]
GO


/*
	Table: 'phpbb_acl_arcade_options'
*/
CREATE TABLE [phpbb_acl_arcade_options] (
	[auth_option_id] [int] IDENTITY (1, 1) NOT NULL ,
	[auth_option] [varchar] (50) DEFAULT ('') NOT NULL ,
	[is_global] [int] DEFAULT (0) NOT NULL ,
	[is_local] [int] DEFAULT (0) NOT NULL ,
	[founder_only] [int] DEFAULT (0) NOT NULL 
) ON [PRIMARY]
GO

ALTER TABLE [phpbb_acl_arcade_options] WITH NOCHECK ADD 
	CONSTRAINT [PK_phpbb_acl_arcade_options] PRIMARY KEY  CLUSTERED 
	(
		[auth_option_id]
	)  ON [PRIMARY] 
GO

CREATE  INDEX [auth_option] ON [phpbb_acl_arcade_options]([auth_option]) ON [PRIMARY]
GO


/*
	Table: 'phpbb_acl_arcade_roles'
*/
CREATE TABLE [phpbb_acl_arcade_roles] (
	[role_id] [int] IDENTITY (1, 1) NOT NULL ,
	[role_name] [varchar] (255) DEFAULT ('') NOT NULL ,
	[role_description] [varchar] (4000) DEFAULT ('') NOT NULL ,
	[role_type] [varchar] (10) DEFAULT ('') NOT NULL ,
	[role_order] [int] DEFAULT (0) NOT NULL 
) ON [PRIMARY]
GO

ALTER TABLE [phpbb_acl_arcade_roles] WITH NOCHECK ADD 
	CONSTRAINT [PK_phpbb_acl_arcade_roles] PRIMARY KEY  CLUSTERED 
	(
		[role_id]
	)  ON [PRIMARY] 
GO

CREATE  INDEX [role_type] ON [phpbb_acl_arcade_roles]([role_type]) ON [PRIMARY]
GO

CREATE  INDEX [role_order] ON [phpbb_acl_arcade_roles]([role_order]) ON [PRIMARY]
GO


/*
	Table: 'phpbb_acl_arcade_roles_data'
*/
CREATE TABLE [phpbb_acl_arcade_roles_data] (
	[role_id] [int] DEFAULT (0) NOT NULL ,
	[auth_option_id] [int] DEFAULT (0) NOT NULL ,
	[auth_setting] [int] DEFAULT (0) NOT NULL 
) ON [PRIMARY]
GO

ALTER TABLE [phpbb_acl_arcade_roles_data] WITH NOCHECK ADD 
	CONSTRAINT [PK_phpbb_acl_arcade_roles_data] PRIMARY KEY  CLUSTERED 
	(
		[role_id],
		[auth_option_id]
	)  ON [PRIMARY] 
GO

CREATE  INDEX [ath_op_id] ON [phpbb_acl_arcade_roles_data]([auth_option_id]) ON [PRIMARY]
GO


/*
	Table: 'phpbb_acl_arcade_users'
*/
CREATE TABLE [phpbb_acl_arcade_users] (
	[user_id] [int] DEFAULT (0) NOT NULL ,
	[cat_id] [int] DEFAULT (0) NOT NULL ,
	[auth_option_id] [int] DEFAULT (0) NOT NULL ,
	[auth_role_id] [int] DEFAULT (0) NOT NULL ,
	[auth_setting] [int] DEFAULT (0) NOT NULL 
) ON [PRIMARY]
GO

CREATE  INDEX [user_id] ON [phpbb_acl_arcade_users]([user_id]) ON [PRIMARY]
GO

CREATE  INDEX [auth_option_id] ON [phpbb_acl_arcade_users]([auth_option_id]) ON [PRIMARY]
GO

CREATE  INDEX [auth_role_id] ON [phpbb_acl_arcade_users]([auth_role_id]) ON [PRIMARY]
GO


/*
	Table: 'phpbb_arcade_access'
*/
CREATE TABLE [phpbb_arcade_access] (
	[cat_id] [int] DEFAULT (0) NOT NULL ,
	[user_id] [int] DEFAULT (0) NOT NULL ,
	[session_id] [char] (32) DEFAULT ('') NOT NULL 
) ON [PRIMARY]
GO

ALTER TABLE [phpbb_arcade_access] WITH NOCHECK ADD 
	CONSTRAINT [PK_phpbb_arcade_access] PRIMARY KEY  CLUSTERED 
	(
		[cat_id],
		[user_id],
		[session_id]
	)  ON [PRIMARY] 
GO


/*
	Table: 'phpbb_arcade_categories'
*/
CREATE TABLE [phpbb_arcade_categories] (
	[cat_id] [int] IDENTITY (1, 1) NOT NULL ,
	[parent_id] [int] DEFAULT (0) NOT NULL ,
	[left_id] [int] DEFAULT (0) NOT NULL ,
	[right_id] [int] DEFAULT (0) NOT NULL ,
	[cat_parents] [text] DEFAULT ('') NOT NULL ,
	[cat_name] [varchar] (255) DEFAULT ('') NOT NULL ,
	[cat_desc] [varchar] (4000) DEFAULT ('') NOT NULL ,
	[cat_desc_bitfield] [varchar] (255) DEFAULT ('') NOT NULL ,
	[cat_desc_options] [int] DEFAULT (7) NOT NULL ,
	[cat_desc_uid] [varchar] (8) DEFAULT ('') NOT NULL ,
	[cat_link] [varchar] (255) DEFAULT ('') NOT NULL ,
	[cat_password] [varchar] (40) DEFAULT ('') NOT NULL ,
	[cat_style] [int] DEFAULT (0) NOT NULL ,
	[cat_display] [int] DEFAULT (0) NOT NULL ,
	[cat_image] [varchar] (255) DEFAULT ('') NOT NULL ,
	[cat_rules] [varchar] (4000) DEFAULT ('') NOT NULL ,
	[cat_rules_link] [varchar] (255) DEFAULT ('') NOT NULL ,
	[cat_rules_bitfield] [varchar] (255) DEFAULT ('') NOT NULL ,
	[cat_rules_options] [int] DEFAULT (7) NOT NULL ,
	[cat_rules_uid] [varchar] (8) DEFAULT ('') NOT NULL ,
	[cat_games_per_page] [int] DEFAULT (0) NOT NULL ,
	[cat_type] [int] DEFAULT (0) NOT NULL ,
	[cat_status] [int] DEFAULT (0) NOT NULL ,
	[cat_games] [int] DEFAULT (0) NOT NULL ,
	[cat_plays] [int] DEFAULT (0) NOT NULL ,
	[cat_download] [int] DEFAULT (1) NOT NULL ,
	[cat_last_play_game_id] [int] DEFAULT (0) NOT NULL ,
	[cat_last_play_game_name] [varchar] (255) DEFAULT (0) NOT NULL ,
	[cat_last_play_user_id] [int] DEFAULT (0) NOT NULL ,
	[cat_last_play_score] [float] DEFAULT (0) NOT NULL ,
	[cat_last_play_time] [int] DEFAULT (0) NOT NULL ,
	[cat_last_game_installdate] [int] DEFAULT (0) NOT NULL ,
	[cat_last_play_username] [varchar] (255) DEFAULT ('') NOT NULL ,
	[cat_last_play_user_colour] [varchar] (6) DEFAULT ('') NOT NULL ,
	[cat_flags] [int] DEFAULT (32) NOT NULL ,
	[display_subcat_list] [int] DEFAULT (1) NOT NULL ,
	[display_on_index] [int] DEFAULT (1) NOT NULL 
) ON [PRIMARY] TEXTIMAGE_ON [PRIMARY]
GO

ALTER TABLE [phpbb_arcade_categories] WITH NOCHECK ADD 
	CONSTRAINT [PK_phpbb_arcade_categories] PRIMARY KEY  CLUSTERED 
	(
		[cat_id]
	)  ON [PRIMARY] 
GO

CREATE  INDEX [left_right_id] ON [phpbb_arcade_categories]([left_id], [right_id]) ON [PRIMARY]
GO

CREATE  INDEX [cat_lastplay_game_id] ON [phpbb_arcade_categories]([cat_last_play_game_id]) ON [PRIMARY]
GO


/*
	Table: 'phpbb_arcade_config'
*/
CREATE TABLE [phpbb_arcade_config] (
	[config_name] [varchar] (255) DEFAULT ('') NOT NULL ,
	[config_value] [varchar] (255) DEFAULT ('') NOT NULL 
) ON [PRIMARY]
GO

ALTER TABLE [phpbb_arcade_config] WITH NOCHECK ADD 
	CONSTRAINT [PK_phpbb_arcade_config] PRIMARY KEY  CLUSTERED 
	(
		[config_name]
	)  ON [PRIMARY] 
GO


/*
	Table: 'phpbb_arcade_errors'
*/
CREATE TABLE [phpbb_arcade_errors] (
	[error_id] [int] IDENTITY (1, 1) NOT NULL ,
	[user_id] [int] DEFAULT (0) NOT NULL ,
	[game_id] [int] DEFAULT (0) NOT NULL ,
	[score] [float] DEFAULT (0) NOT NULL ,
	[error_date] [int] DEFAULT (0) NOT NULL ,
	[error_type] [int] DEFAULT (0) NOT NULL ,
	[game_type] [int] DEFAULT (0) NOT NULL ,
	[submitted_game_type] [int] DEFAULT (0) NOT NULL ,
	[played_time] [int] DEFAULT (0) NOT NULL ,
	[error_ip] [varchar] (40) DEFAULT ('') NOT NULL 
) ON [PRIMARY]
GO

ALTER TABLE [phpbb_arcade_errors] WITH NOCHECK ADD 
	CONSTRAINT [PK_phpbb_arcade_errors] PRIMARY KEY  CLUSTERED 
	(
		[error_id]
	)  ON [PRIMARY] 
GO


/*
	Table: 'phpbb_arcade_favorites'
*/
CREATE TABLE [phpbb_arcade_favorites] (
	[user_id] [int] DEFAULT (0) NOT NULL ,
	[game_id] [int] DEFAULT (0) NOT NULL 
) ON [PRIMARY]
GO

ALTER TABLE [phpbb_arcade_favorites] WITH NOCHECK ADD 
	CONSTRAINT [PK_phpbb_arcade_favorites] PRIMARY KEY  CLUSTERED 
	(
		[user_id],
		[game_id]
	)  ON [PRIMARY] 
GO


/*
	Table: 'phpbb_arcade_games'
*/
CREATE TABLE [phpbb_arcade_games] (
	[game_id] [int] IDENTITY (1, 1) NOT NULL ,
	[game_image] [varchar] (255) DEFAULT ('') NOT NULL ,
	[game_desc] [varchar] (8000) DEFAULT ('') NOT NULL ,
	[game_highscore] [float] DEFAULT (0) NOT NULL ,
	[game_highdate] [int] DEFAULT (0) NOT NULL ,
	[game_highuser] [int] DEFAULT (0) NOT NULL ,
	[game_files] [text] DEFAULT ('') NOT NULL ,
	[game_name] [varchar] (255) DEFAULT ('') NOT NULL ,
	[game_name_clean] [varchar] (255) DEFAULT ('') NOT NULL ,
	[game_swf] [varchar] (255) DEFAULT ('') NOT NULL ,
	[game_scorevar] [varchar] (255) DEFAULT ('') NOT NULL ,
	[game_type] [int] DEFAULT (0) NOT NULL ,
	[game_width] [int] DEFAULT (550) NOT NULL ,
	[game_height] [int] DEFAULT (380) NOT NULL ,
	[game_installdate] [int] DEFAULT (0) NOT NULL ,
	[game_filesize] [int] DEFAULT (0) NOT NULL ,
	[game_scoretype] [int] DEFAULT (0) NOT NULL ,
	[game_order] [int] DEFAULT (0) NOT NULL ,
	[game_plays] [int] DEFAULT (0) NOT NULL ,
	[game_votetotal] [int] DEFAULT (0) NOT NULL ,
	[game_votesum] [int] DEFAULT (0) NOT NULL ,
	[game_download_total] [int] DEFAULT (0) NOT NULL ,
	[game_download] [int] DEFAULT (1) NOT NULL ,
	[cat_id] [int] DEFAULT (0) NOT NULL 
) ON [PRIMARY] TEXTIMAGE_ON [PRIMARY]
GO

ALTER TABLE [phpbb_arcade_games] WITH NOCHECK ADD 
	CONSTRAINT [PK_phpbb_arcade_games] PRIMARY KEY  CLUSTERED 
	(
		[game_id]
	)  ON [PRIMARY] 
GO


/*
	Table: 'phpbb_arcade_game_rating'
*/
CREATE TABLE [phpbb_arcade_game_rating] (
	[game_id] [int] DEFAULT (0) NOT NULL ,
	[user_id] [int] DEFAULT (0) NOT NULL ,
	[game_rating] [int] DEFAULT (0) NOT NULL ,
	[rating_date] [int] DEFAULT (0) NOT NULL ,
	[user_ip] [varchar] (40) DEFAULT ('') NOT NULL 
) ON [PRIMARY]
GO

ALTER TABLE [phpbb_arcade_game_rating] WITH NOCHECK ADD 
	CONSTRAINT [PK_phpbb_arcade_game_rating] PRIMARY KEY  CLUSTERED 
	(
		[game_id],
		[user_id]
	)  ON [PRIMARY] 
GO


/*
	Table: 'phpbb_arcade_scores'
*/
CREATE TABLE [phpbb_arcade_scores] (
	[game_id] [int] DEFAULT (0) NOT NULL ,
	[user_id] [int] DEFAULT (0) NOT NULL ,
	[score] [float] DEFAULT (0) NOT NULL ,
	[comment_text] [text] DEFAULT ('') NOT NULL ,
	[comment_bitfield] [varchar] (255) DEFAULT ('') NOT NULL ,
	[comment_options] [int] DEFAULT (7) NOT NULL ,
	[comment_uid] [varchar] (8) DEFAULT ('') NOT NULL ,
	[score_date] [int] DEFAULT (0) NOT NULL ,
	[total_time] [int] DEFAULT (0) NOT NULL ,
	[total_plays] [int] DEFAULT (0) NOT NULL 
) ON [PRIMARY] TEXTIMAGE_ON [PRIMARY]
GO

ALTER TABLE [phpbb_arcade_scores] WITH NOCHECK ADD 
	CONSTRAINT [PK_phpbb_arcade_scores] PRIMARY KEY  CLUSTERED 
	(
		[game_id],
		[user_id]
	)  ON [PRIMARY] 
GO


/*
	Table: 'phpbb_arcade_sessions'
*/
CREATE TABLE [phpbb_arcade_sessions] (
	[session_id] [char] (32) DEFAULT ('') NOT NULL ,
	[game_id] [int] DEFAULT (0) NOT NULL ,
	[user_id] [int] DEFAULT (0) NOT NULL ,
	[game_type] [int] DEFAULT (0) NOT NULL ,
	[randchar1] [varchar] (30) DEFAULT ('') NOT NULL ,
	[randchar2] [varchar] (30) DEFAULT ('') NOT NULL ,
	[randgid1] [varchar] (30) DEFAULT ('') NOT NULL ,
	[randgid2] [varchar] (30) DEFAULT ('') NOT NULL ,
	[start_time] [int] DEFAULT (0) NOT NULL 
) ON [PRIMARY]
GO

ALTER TABLE [phpbb_arcade_sessions] WITH NOCHECK ADD 
	CONSTRAINT [PK_phpbb_arcade_sessions] PRIMARY KEY  CLUSTERED 
	(
		[session_id]
	)  ON [PRIMARY] 
GO


/*
	Table: 'phpbb_arcade_download'
*/
CREATE TABLE [phpbb_arcade_download] (
	[user_id] [int] DEFAULT (0) NOT NULL ,
	[game_id] [int] DEFAULT (0) NOT NULL ,
	[total] [int] DEFAULT (0) NOT NULL ,
	[download_time] [int] DEFAULT (0) NOT NULL 
) ON [PRIMARY]
GO

ALTER TABLE [phpbb_arcade_download] WITH NOCHECK ADD 
	CONSTRAINT [PK_phpbb_arcade_download] PRIMARY KEY  CLUSTERED 
	(
		[user_id],
		[game_id]
	)  ON [PRIMARY] 
GO



COMMIT
GO

