/*

 $Id: $

*/

/*
  This first section is optional, however its probably the best method
  of running phpBB on Oracle. If you already have a tablespace and user created
  for phpBB you can leave this section commented out!

  The first set of statements create a phpBB tablespace and a phpBB user,
  make sure you change the password of the phpBB user before you run this script!!
*/

/*
CREATE TABLESPACE "PHPBB"
	LOGGING
	DATAFILE 'E:\ORACLE\ORADATA\LOCAL\PHPBB.ora'
	SIZE 10M
	AUTOEXTEND ON NEXT 10M
	MAXSIZE 100M;

CREATE USER "PHPBB"
	PROFILE "DEFAULT"
	IDENTIFIED BY "phpbb_password"
	DEFAULT TABLESPACE "PHPBB"
	QUOTA UNLIMITED ON "PHPBB"
	ACCOUNT UNLOCK;

GRANT ANALYZE ANY TO "PHPBB";
GRANT CREATE SEQUENCE TO "PHPBB";
GRANT CREATE SESSION TO "PHPBB";
GRANT CREATE TABLE TO "PHPBB";
GRANT CREATE TRIGGER TO "PHPBB";
GRANT CREATE VIEW TO "PHPBB";
GRANT "CONNECT" TO "PHPBB";

COMMIT;
DISCONNECT;

CONNECT phpbb/phpbb_password;
*/
/*
	Table: 'phpbb_acl_arcade_groups'
*/
CREATE TABLE phpbb_acl_arcade_groups (
	group_id number(8) DEFAULT '0' NOT NULL,
	cat_id number(8) DEFAULT '0' NOT NULL,
	auth_option_id number(8) DEFAULT '0' NOT NULL,
	auth_role_id number(8) DEFAULT '0' NOT NULL,
	auth_setting number(2) DEFAULT '0' NOT NULL
)
/

CREATE INDEX phpbb_acl_arcade_groups_group_id ON phpbb_acl_arcade_groups (group_id)
/
CREATE INDEX phpbb_acl_arcade_groups_auth_opt_id ON phpbb_acl_arcade_groups (auth_option_id)
/
CREATE INDEX phpbb_acl_arcade_groups_auth_role_id ON phpbb_acl_arcade_groups (auth_role_id)
/

/*
	Table: 'phpbb_acl_arcade_options'
*/
CREATE TABLE phpbb_acl_arcade_options (
	auth_option_id number(8) NOT NULL,
	auth_option varchar2(50) DEFAULT '' ,
	is_global number(1) DEFAULT '0' NOT NULL,
	is_local number(1) DEFAULT '0' NOT NULL,
	founder_only number(1) DEFAULT '0' NOT NULL,
	CONSTRAINT pk_phpbb_acl_arcade_options PRIMARY KEY (auth_option_id)
)
/

CREATE INDEX phpbb_acl_arcade_options_auth_option ON phpbb_acl_arcade_options (auth_option)
/

CREATE SEQUENCE phpbb_acl_arcade_options_seq
/

CREATE OR REPLACE TRIGGER t_phpbb_acl_arcade_options
BEFORE INSERT ON phpbb_acl_arcade_options
FOR EACH ROW WHEN (
	new.auth_option_id IS NULL OR new.auth_option_id = 0
)
BEGIN
	SELECT phpbb_acl_arcade_options_seq.nextval
	INTO :new.auth_option_id
	FROM dual;
END;
/


/*
	Table: 'phpbb_acl_arcade_roles'
*/
CREATE TABLE phpbb_acl_arcade_roles (
	role_id number(8) NOT NULL,
	role_name varchar2(765) DEFAULT '' ,
	role_description clob DEFAULT '' ,
	role_type varchar2(10) DEFAULT '' ,
	role_order number(4) DEFAULT '0' NOT NULL,
	CONSTRAINT pk_phpbb_acl_arcade_roles PRIMARY KEY (role_id)
)
/

CREATE INDEX phpbb_acl_arcade_roles_role_type ON phpbb_acl_arcade_roles (role_type)
/
CREATE INDEX phpbb_acl_arcade_roles_role_order ON phpbb_acl_arcade_roles (role_order)
/

CREATE SEQUENCE phpbb_acl_arcade_roles_seq
/

CREATE OR REPLACE TRIGGER t_phpbb_acl_arcade_roles
BEFORE INSERT ON phpbb_acl_arcade_roles
FOR EACH ROW WHEN (
	new.role_id IS NULL OR new.role_id = 0
)
BEGIN
	SELECT phpbb_acl_arcade_roles_seq.nextval
	INTO :new.role_id
	FROM dual;
END;
/


/*
	Table: 'phpbb_acl_arcade_roles_data'
*/
CREATE TABLE phpbb_acl_arcade_roles_data (
	role_id number(8) DEFAULT '0' NOT NULL,
	auth_option_id number(8) DEFAULT '0' NOT NULL,
	auth_setting number(2) DEFAULT '0' NOT NULL,
	CONSTRAINT pk_phpbb_acl_arcade_roles_data PRIMARY KEY (role_id, auth_option_id)
)
/

CREATE INDEX phpbb_acl_arcade_roles_data_ath_op_id ON phpbb_acl_arcade_roles_data (auth_option_id)
/

/*
	Table: 'phpbb_acl_arcade_users'
*/
CREATE TABLE phpbb_acl_arcade_users (
	user_id number(8) DEFAULT '0' NOT NULL,
	cat_id number(8) DEFAULT '0' NOT NULL,
	auth_option_id number(8) DEFAULT '0' NOT NULL,
	auth_role_id number(8) DEFAULT '0' NOT NULL,
	auth_setting number(2) DEFAULT '0' NOT NULL
)
/

CREATE INDEX phpbb_acl_arcade_users_user_id ON phpbb_acl_arcade_users (user_id)
/
CREATE INDEX phpbb_acl_arcade_users_auth_option_id ON phpbb_acl_arcade_users (auth_option_id)
/
CREATE INDEX phpbb_acl_arcade_users_auth_role_id ON phpbb_acl_arcade_users (auth_role_id)
/

/*
	Table: 'phpbb_arcade_access'
*/
CREATE TABLE phpbb_arcade_access (
	cat_id number(8) DEFAULT '0' NOT NULL,
	user_id number(8) DEFAULT '0' NOT NULL,
	session_id char(32) DEFAULT '' ,
	CONSTRAINT pk_phpbb_arcade_access PRIMARY KEY (cat_id, user_id, session_id)
)
/


/*
	Table: 'phpbb_arcade_categories'
*/
CREATE TABLE phpbb_arcade_categories (
	cat_id number(8) NOT NULL,
	parent_id number(8) DEFAULT '0' NOT NULL,
	left_id number(8) DEFAULT '0' NOT NULL,
	right_id number(8) DEFAULT '0' NOT NULL,
	cat_parents clob DEFAULT '' ,
	cat_name varchar2(765) DEFAULT '' ,
	cat_desc clob DEFAULT '' ,
	cat_desc_bitfield varchar2(255) DEFAULT '' ,
	cat_desc_options number(11) DEFAULT '7' NOT NULL,
	cat_desc_uid varchar2(8) DEFAULT '' ,
	cat_link varchar2(765) DEFAULT '' ,
	cat_password varchar2(120) DEFAULT '' ,
	cat_style number(4) DEFAULT '0' NOT NULL,
	cat_display number(4) DEFAULT '0' NOT NULL,
	cat_image varchar2(255) DEFAULT '' ,
	cat_rules clob DEFAULT '' ,
	cat_rules_link varchar2(765) DEFAULT '' ,
	cat_rules_bitfield varchar2(255) DEFAULT '' ,
	cat_rules_options number(11) DEFAULT '7' NOT NULL,
	cat_rules_uid varchar2(8) DEFAULT '' ,
	cat_games_per_page number(4) DEFAULT '0' NOT NULL,
	cat_type number(4) DEFAULT '0' NOT NULL,
	cat_status number(4) DEFAULT '0' NOT NULL,
	cat_games number(8) DEFAULT '0' NOT NULL,
	cat_plays number(8) DEFAULT '0' NOT NULL,
	cat_download number(1) DEFAULT '1' NOT NULL,
	cat_last_play_game_id number(8) DEFAULT '0' NOT NULL,
	cat_last_play_game_name varchar2(255) DEFAULT '0' NOT NULL,
	cat_last_play_user_id number(8) DEFAULT '0' NOT NULL,
	cat_last_play_score number(25, 3) DEFAULT '0' NOT NULL,
	cat_last_play_time number(11) DEFAULT '0' NOT NULL,
	cat_last_game_installdate number(11) DEFAULT '0' NOT NULL,
	cat_last_play_username varchar2(765) DEFAULT '' ,
	cat_last_play_user_colour varchar2(6) DEFAULT '' ,
	cat_flags number(4) DEFAULT '32' NOT NULL,
	display_subcat_list number(1) DEFAULT '1' NOT NULL,
	display_on_index number(1) DEFAULT '1' NOT NULL,
	CONSTRAINT pk_phpbb_arcade_categories PRIMARY KEY (cat_id)
)
/

CREATE INDEX phpbb_arcade_categories_left_right_id ON phpbb_arcade_categories (left_id, right_id)
/
CREATE INDEX phpbb_arcade_categories_cat_lastplay_game_id ON phpbb_arcade_categories (cat_last_play_game_id)
/

CREATE SEQUENCE phpbb_arcade_categories_seq
/

CREATE OR REPLACE TRIGGER t_phpbb_arcade_categories
BEFORE INSERT ON phpbb_arcade_categories
FOR EACH ROW WHEN (
	new.cat_id IS NULL OR new.cat_id = 0
)
BEGIN
	SELECT phpbb_arcade_categories_seq.nextval
	INTO :new.cat_id
	FROM dual;
END;
/


/*
	Table: 'phpbb_arcade_config'
*/
CREATE TABLE phpbb_arcade_config (
	config_name varchar2(255) DEFAULT '' ,
	config_value varchar2(765) DEFAULT '' ,
	CONSTRAINT pk_phpbb_arcade_config PRIMARY KEY (config_name)
)
/


/*
	Table: 'phpbb_arcade_errors'
*/
CREATE TABLE phpbb_arcade_errors (
	error_id number(8) NOT NULL,
	user_id number(8) DEFAULT '0' NOT NULL,
	game_id number(8) DEFAULT '0' NOT NULL,
	score number(25, 3) DEFAULT '0' NOT NULL,
	error_date number(11) DEFAULT '0' NOT NULL,
	error_type number(1) DEFAULT '0' NOT NULL,
	game_type number(1) DEFAULT '0' NOT NULL,
	submitted_game_type number(1) DEFAULT '0' NOT NULL,
	played_time number(11) DEFAULT '0' NOT NULL,
	error_ip varchar2(40) DEFAULT '' ,
	CONSTRAINT pk_phpbb_arcade_errors PRIMARY KEY (error_id)
)
/


CREATE SEQUENCE phpbb_arcade_errors_seq
/

CREATE OR REPLACE TRIGGER t_phpbb_arcade_errors
BEFORE INSERT ON phpbb_arcade_errors
FOR EACH ROW WHEN (
	new.error_id IS NULL OR new.error_id = 0
)
BEGIN
	SELECT phpbb_arcade_errors_seq.nextval
	INTO :new.error_id
	FROM dual;
END;
/


/*
	Table: 'phpbb_arcade_favorites'
*/
CREATE TABLE phpbb_arcade_favorites (
	user_id number(8) DEFAULT '0' NOT NULL,
	game_id number(8) DEFAULT '0' NOT NULL,
	CONSTRAINT pk_phpbb_arcade_favorites PRIMARY KEY (user_id, game_id)
)
/


/*
	Table: 'phpbb_arcade_games'
*/
CREATE TABLE phpbb_arcade_games (
	game_id number(8) NOT NULL,
	game_image varchar2(255) DEFAULT '' ,
	game_desc clob DEFAULT '' ,
	game_highscore number(25, 3) DEFAULT '0' NOT NULL,
	game_highdate number(11) DEFAULT '0' NOT NULL,
	game_highuser number(8) DEFAULT '0' NOT NULL,
	game_files clob DEFAULT '' ,
	game_name varchar2(255) DEFAULT '' ,
	game_name_clean varchar2(255) DEFAULT '' ,
	game_swf varchar2(255) DEFAULT '' ,
	game_scorevar varchar2(255) DEFAULT '' ,
	game_type number(1) DEFAULT '0' NOT NULL,
	game_width number(5) DEFAULT '550' NOT NULL,
	game_height number(5) DEFAULT '380' NOT NULL,
	game_installdate number(11) DEFAULT '0' NOT NULL,
	game_filesize number(20) DEFAULT '0' NOT NULL,
	game_scoretype number(1) DEFAULT '0' NOT NULL,
	game_order number(8) DEFAULT '0' NOT NULL,
	game_plays number(8) DEFAULT '0' NOT NULL,
	game_votetotal number(8) DEFAULT '0' NOT NULL,
	game_votesum number(8) DEFAULT '0' NOT NULL,
	game_download_total number(8) DEFAULT '0' NOT NULL,
	game_download number(1) DEFAULT '1' NOT NULL,
	cat_id number(8) DEFAULT '0' NOT NULL,
	CONSTRAINT pk_phpbb_arcade_games PRIMARY KEY (game_id)
)
/


CREATE SEQUENCE phpbb_arcade_games_seq
/

CREATE OR REPLACE TRIGGER t_phpbb_arcade_games
BEFORE INSERT ON phpbb_arcade_games
FOR EACH ROW WHEN (
	new.game_id IS NULL OR new.game_id = 0
)
BEGIN
	SELECT phpbb_arcade_games_seq.nextval
	INTO :new.game_id
	FROM dual;
END;
/


/*
	Table: 'phpbb_arcade_game_rating'
*/
CREATE TABLE phpbb_arcade_game_rating (
	game_id number(8) DEFAULT '0' NOT NULL,
	user_id number(8) DEFAULT '0' NOT NULL,
	game_rating number(8) DEFAULT '0' NOT NULL,
	rating_date number(11) DEFAULT '0' NOT NULL,
	user_ip varchar2(40) DEFAULT '' ,
	CONSTRAINT pk_phpbb_arcade_game_rating PRIMARY KEY (game_id, user_id)
)
/


/*
	Table: 'phpbb_arcade_scores'
*/
CREATE TABLE phpbb_arcade_scores (
	game_id number(8) DEFAULT '0' NOT NULL,
	user_id number(8) DEFAULT '0' NOT NULL,
	score number(25, 3) DEFAULT '0' NOT NULL,
	comment_text clob DEFAULT '' ,
	comment_bitfield varchar2(255) DEFAULT '' ,
	comment_options number(11) DEFAULT '7' NOT NULL,
	comment_uid varchar2(8) DEFAULT '' ,
	score_date number(11) DEFAULT '0' NOT NULL,
	total_time number(11) DEFAULT '0' NOT NULL,
	total_plays number(8) DEFAULT '0' NOT NULL,
	CONSTRAINT pk_phpbb_arcade_scores PRIMARY KEY (game_id, user_id)
)
/


/*
	Table: 'phpbb_arcade_sessions'
*/
CREATE TABLE phpbb_arcade_sessions (
	session_id char(32) DEFAULT '' ,
	game_id number(8) DEFAULT '0' NOT NULL,
	user_id number(8) DEFAULT '0' NOT NULL,
	game_type number(1) DEFAULT '0' NOT NULL,
	randchar1 varchar2(30) DEFAULT '' ,
	randchar2 varchar2(30) DEFAULT '' ,
	randgid1 varchar2(30) DEFAULT '' ,
	randgid2 varchar2(30) DEFAULT '' ,
	start_time number(11) DEFAULT '0' NOT NULL,
	CONSTRAINT pk_phpbb_arcade_sessions PRIMARY KEY (session_id)
)
/


/*
	Table: 'phpbb_arcade_download'
*/
CREATE TABLE phpbb_arcade_download (
	user_id number(8) DEFAULT '0' NOT NULL,
	game_id number(8) DEFAULT '0' NOT NULL,
	total number(8) DEFAULT '0' NOT NULL,
	download_time number(11) DEFAULT '0' NOT NULL,
	CONSTRAINT pk_phpbb_arcade_download PRIMARY KEY (user_id, game_id)
)
/


