/*

 $Id: $

*/

BEGIN;

/*
	Domain definition
*/

/*
CREATE DOMAIN varchar_ci AS varchar(255) NOT NULL DEFAULT ''::character varying;
*/

/*
	Operation Functions
*/

/*
CREATE FUNCTION _varchar_ci_equal(varchar_ci, varchar_ci) RETURNS boolean AS 'SELECT LOWER($1) = LOWER($2)' LANGUAGE SQL STRICT;
CREATE FUNCTION _varchar_ci_not_equal(varchar_ci, varchar_ci) RETURNS boolean AS 'SELECT LOWER($1) != LOWER($2)' LANGUAGE SQL STRICT;
CREATE FUNCTION _varchar_ci_less_than(varchar_ci, varchar_ci) RETURNS boolean AS 'SELECT LOWER($1) < LOWER($2)' LANGUAGE SQL STRICT;
CREATE FUNCTION _varchar_ci_less_equal(varchar_ci, varchar_ci) RETURNS boolean AS 'SELECT LOWER($1) <= LOWER($2)' LANGUAGE SQL STRICT;
CREATE FUNCTION _varchar_ci_greater_than(varchar_ci, varchar_ci) RETURNS boolean AS 'SELECT LOWER($1) > LOWER($2)' LANGUAGE SQL STRICT;
CREATE FUNCTION _varchar_ci_greater_equals(varchar_ci, varchar_ci) RETURNS boolean AS 'SELECT LOWER($1) >= LOWER($2)' LANGUAGE SQL STRICT;
*/

/*
	Operators
*/
/*
CREATE OPERATOR <(
  PROCEDURE = _varchar_ci_less_than,
  LEFTARG = varchar_ci,
  RIGHTARG = varchar_ci,
  COMMUTATOR = >,
  NEGATOR = >=,
  RESTRICT = scalarltsel,
  JOIN = scalarltjoinsel);

CREATE OPERATOR <=(
  PROCEDURE = _varchar_ci_less_equal,
  LEFTARG = varchar_ci,
  RIGHTARG = varchar_ci,
  COMMUTATOR = >=,
  NEGATOR = >,
  RESTRICT = scalarltsel,
  JOIN = scalarltjoinsel);

CREATE OPERATOR >(
  PROCEDURE = _varchar_ci_greater_than,
  LEFTARG = varchar_ci,
  RIGHTARG = varchar_ci,
  COMMUTATOR = <,
  NEGATOR = <=,
  RESTRICT = scalargtsel,
  JOIN = scalargtjoinsel);

CREATE OPERATOR >=(
  PROCEDURE = _varchar_ci_greater_equals,
  LEFTARG = varchar_ci,
  RIGHTARG = varchar_ci,
  COMMUTATOR = <=,
  NEGATOR = <,
  RESTRICT = scalargtsel,
  JOIN = scalargtjoinsel);

CREATE OPERATOR <>(
  PROCEDURE = _varchar_ci_not_equal,
  LEFTARG = varchar_ci,
  RIGHTARG = varchar_ci,
  COMMUTATOR = <>,
  NEGATOR = =,
  RESTRICT = neqsel,
  JOIN = neqjoinsel);

CREATE OPERATOR =(
  PROCEDURE = _varchar_ci_equal,
  LEFTARG = varchar_ci,
  RIGHTARG = varchar_ci,
  COMMUTATOR = =,
  NEGATOR = <>,
  RESTRICT = eqsel,
  JOIN = eqjoinsel,
  HASHES,
  MERGES,
  SORT1= <);
*/
/*
	Table: 'phpbb_acl_arcade_groups'
*/
CREATE TABLE phpbb_acl_arcade_groups (
	group_id INT4 DEFAULT '0' NOT NULL CHECK (group_id >= 0),
	cat_id INT4 DEFAULT '0' NOT NULL CHECK (cat_id >= 0),
	auth_option_id INT4 DEFAULT '0' NOT NULL CHECK (auth_option_id >= 0),
	auth_role_id INT4 DEFAULT '0' NOT NULL CHECK (auth_role_id >= 0),
	auth_setting INT2 DEFAULT '0' NOT NULL
);

CREATE INDEX phpbb_acl_arcade_groups_group_id ON phpbb_acl_arcade_groups (group_id);
CREATE INDEX phpbb_acl_arcade_groups_auth_opt_id ON phpbb_acl_arcade_groups (auth_option_id);
CREATE INDEX phpbb_acl_arcade_groups_auth_role_id ON phpbb_acl_arcade_groups (auth_role_id);

/*
	Table: 'phpbb_acl_arcade_options'
*/
CREATE SEQUENCE phpbb_acl_arcade_options_seq;

CREATE TABLE phpbb_acl_arcade_options (
	auth_option_id INT4 DEFAULT nextval('phpbb_acl_arcade_options_seq'),
	auth_option varchar(50) DEFAULT '' NOT NULL,
	is_global INT2 DEFAULT '0' NOT NULL CHECK (is_global >= 0),
	is_local INT2 DEFAULT '0' NOT NULL CHECK (is_local >= 0),
	founder_only INT2 DEFAULT '0' NOT NULL CHECK (founder_only >= 0),
	PRIMARY KEY (auth_option_id)
);

CREATE INDEX phpbb_acl_arcade_options_auth_option ON phpbb_acl_arcade_options (auth_option);

/*
	Table: 'phpbb_acl_arcade_roles'
*/
CREATE SEQUENCE phpbb_acl_arcade_roles_seq;

CREATE TABLE phpbb_acl_arcade_roles (
	role_id INT4 DEFAULT nextval('phpbb_acl_arcade_roles_seq'),
	role_name varchar(255) DEFAULT '' NOT NULL,
	role_description varchar(4000) DEFAULT '' NOT NULL,
	role_type varchar(10) DEFAULT '' NOT NULL,
	role_order INT2 DEFAULT '0' NOT NULL CHECK (role_order >= 0),
	PRIMARY KEY (role_id)
);

CREATE INDEX phpbb_acl_arcade_roles_role_type ON phpbb_acl_arcade_roles (role_type);
CREATE INDEX phpbb_acl_arcade_roles_role_order ON phpbb_acl_arcade_roles (role_order);

/*
	Table: 'phpbb_acl_arcade_roles_data'
*/
CREATE TABLE phpbb_acl_arcade_roles_data (
	role_id INT4 DEFAULT '0' NOT NULL CHECK (role_id >= 0),
	auth_option_id INT4 DEFAULT '0' NOT NULL CHECK (auth_option_id >= 0),
	auth_setting INT2 DEFAULT '0' NOT NULL,
	PRIMARY KEY (role_id, auth_option_id)
);

CREATE INDEX phpbb_acl_arcade_roles_data_ath_op_id ON phpbb_acl_arcade_roles_data (auth_option_id);

/*
	Table: 'phpbb_acl_arcade_users'
*/
CREATE TABLE phpbb_acl_arcade_users (
	user_id INT4 DEFAULT '0' NOT NULL CHECK (user_id >= 0),
	cat_id INT4 DEFAULT '0' NOT NULL CHECK (cat_id >= 0),
	auth_option_id INT4 DEFAULT '0' NOT NULL CHECK (auth_option_id >= 0),
	auth_role_id INT4 DEFAULT '0' NOT NULL CHECK (auth_role_id >= 0),
	auth_setting INT2 DEFAULT '0' NOT NULL
);

CREATE INDEX phpbb_acl_arcade_users_user_id ON phpbb_acl_arcade_users (user_id);
CREATE INDEX phpbb_acl_arcade_users_auth_option_id ON phpbb_acl_arcade_users (auth_option_id);
CREATE INDEX phpbb_acl_arcade_users_auth_role_id ON phpbb_acl_arcade_users (auth_role_id);

/*
	Table: 'phpbb_arcade_access'
*/
CREATE TABLE phpbb_arcade_access (
	cat_id INT4 DEFAULT '0' NOT NULL CHECK (cat_id >= 0),
	user_id INT4 DEFAULT '0' NOT NULL CHECK (user_id >= 0),
	session_id char(32) DEFAULT '' NOT NULL,
	PRIMARY KEY (cat_id, user_id, session_id)
);


/*
	Table: 'phpbb_arcade_categories'
*/
CREATE SEQUENCE phpbb_arcade_categories_seq;

CREATE TABLE phpbb_arcade_categories (
	cat_id INT4 DEFAULT nextval('phpbb_arcade_categories_seq'),
	parent_id INT4 DEFAULT '0' NOT NULL CHECK (parent_id >= 0),
	left_id INT4 DEFAULT '0' NOT NULL CHECK (left_id >= 0),
	right_id INT4 DEFAULT '0' NOT NULL CHECK (right_id >= 0),
	cat_parents TEXT DEFAULT '' NOT NULL,
	cat_name varchar(255) DEFAULT '' NOT NULL,
	cat_desc varchar(4000) DEFAULT '' NOT NULL,
	cat_desc_bitfield varchar(255) DEFAULT '' NOT NULL,
	cat_desc_options INT4 DEFAULT '7' NOT NULL CHECK (cat_desc_options >= 0),
	cat_desc_uid varchar(8) DEFAULT '' NOT NULL,
	cat_link varchar(255) DEFAULT '' NOT NULL,
	cat_password varchar(40) DEFAULT '' NOT NULL,
	cat_style INT2 DEFAULT '0' NOT NULL CHECK (cat_style >= 0),
	cat_display INT2 DEFAULT '0' NOT NULL CHECK (cat_display >= 0),
	cat_image varchar(255) DEFAULT '' NOT NULL,
	cat_rules varchar(4000) DEFAULT '' NOT NULL,
	cat_rules_link varchar(255) DEFAULT '' NOT NULL,
	cat_rules_bitfield varchar(255) DEFAULT '' NOT NULL,
	cat_rules_options INT4 DEFAULT '7' NOT NULL CHECK (cat_rules_options >= 0),
	cat_rules_uid varchar(8) DEFAULT '' NOT NULL,
	cat_games_per_page INT2 DEFAULT '0' NOT NULL,
	cat_type INT2 DEFAULT '0' NOT NULL,
	cat_status INT2 DEFAULT '0' NOT NULL,
	cat_games INT4 DEFAULT '0' NOT NULL CHECK (cat_games >= 0),
	cat_plays INT4 DEFAULT '0' NOT NULL CHECK (cat_plays >= 0),
	cat_download INT2 DEFAULT '1' NOT NULL,
	cat_last_play_game_id INT4 DEFAULT '0' NOT NULL CHECK (cat_last_play_game_id >= 0),
	cat_last_play_game_name varchar(255) DEFAULT '0' NOT NULL,
	cat_last_play_user_id INT4 DEFAULT '0' NOT NULL CHECK (cat_last_play_user_id >= 0),
	cat_last_play_score decimal(25,3) DEFAULT '0' NOT NULL,
	cat_last_play_time INT4 DEFAULT '0' NOT NULL CHECK (cat_last_play_time >= 0),
	cat_last_game_installdate INT4 DEFAULT '0' NOT NULL CHECK (cat_last_game_installdate >= 0),
	cat_last_play_username varchar(255) DEFAULT '' NOT NULL,
	cat_last_play_user_colour varchar(6) DEFAULT '' NOT NULL,
	cat_flags INT2 DEFAULT '32' NOT NULL,
	display_subcat_list INT2 DEFAULT '1' NOT NULL CHECK (display_subcat_list >= 0),
	display_on_index INT2 DEFAULT '1' NOT NULL CHECK (display_on_index >= 0),
	PRIMARY KEY (cat_id)
);

CREATE INDEX phpbb_arcade_categories_left_right_id ON phpbb_arcade_categories (left_id, right_id);
CREATE INDEX phpbb_arcade_categories_cat_lastplay_game_id ON phpbb_arcade_categories (cat_last_play_game_id);

/*
	Table: 'phpbb_arcade_config'
*/
CREATE TABLE phpbb_arcade_config (
	config_name varchar(255) DEFAULT '' NOT NULL,
	config_value varchar(255) DEFAULT '' NOT NULL,
	PRIMARY KEY (config_name)
);


/*
	Table: 'phpbb_arcade_errors'
*/
CREATE SEQUENCE phpbb_arcade_errors_seq;

CREATE TABLE phpbb_arcade_errors (
	error_id INT4 DEFAULT nextval('phpbb_arcade_errors_seq'),
	user_id INT4 DEFAULT '0' NOT NULL CHECK (user_id >= 0),
	game_id INT4 DEFAULT '0' NOT NULL CHECK (game_id >= 0),
	score decimal(25,3) DEFAULT '0' NOT NULL,
	error_date INT4 DEFAULT '0' NOT NULL CHECK (error_date >= 0),
	error_type INT2 DEFAULT '0' NOT NULL,
	game_type INT2 DEFAULT '0' NOT NULL,
	submitted_game_type INT2 DEFAULT '0' NOT NULL,
	played_time INT4 DEFAULT '0' NOT NULL,
	error_ip varchar(40) DEFAULT '' NOT NULL,
	PRIMARY KEY (error_id)
);


/*
	Table: 'phpbb_arcade_favorites'
*/
CREATE TABLE phpbb_arcade_favorites (
	user_id INT4 DEFAULT '0' NOT NULL CHECK (user_id >= 0),
	game_id INT4 DEFAULT '0' NOT NULL CHECK (game_id >= 0),
	PRIMARY KEY (user_id, game_id)
);


/*
	Table: 'phpbb_arcade_games'
*/
CREATE SEQUENCE phpbb_arcade_games_seq;

CREATE TABLE phpbb_arcade_games (
	game_id INT4 DEFAULT nextval('phpbb_arcade_games_seq'),
	game_image varchar(255) DEFAULT '' NOT NULL,
	game_desc varchar(8000) DEFAULT '' NOT NULL,
	game_highscore decimal(25,3) DEFAULT '0' NOT NULL,
	game_highdate INT4 DEFAULT '0' NOT NULL CHECK (game_highdate >= 0),
	game_highuser INT4 DEFAULT '0' NOT NULL CHECK (game_highuser >= 0),
	game_files TEXT DEFAULT '' NOT NULL,
	game_name varchar(255) DEFAULT '' NOT NULL,
	game_name_clean varchar(255) DEFAULT '' NOT NULL,
	game_swf varchar(255) DEFAULT '' NOT NULL,
	game_scorevar varchar(255) DEFAULT '' NOT NULL,
	game_type INT2 DEFAULT '0' NOT NULL,
	game_width INT4 DEFAULT '550' NOT NULL CHECK (game_width >= 0),
	game_height INT4 DEFAULT '380' NOT NULL CHECK (game_height >= 0),
	game_installdate INT4 DEFAULT '0' NOT NULL CHECK (game_installdate >= 0),
	game_filesize INT4 DEFAULT '0' NOT NULL CHECK (game_filesize >= 0),
	game_scoretype INT2 DEFAULT '0' NOT NULL,
	game_order INT4 DEFAULT '0' NOT NULL CHECK (game_order >= 0),
	game_plays INT4 DEFAULT '0' NOT NULL CHECK (game_plays >= 0),
	game_votetotal INT4 DEFAULT '0' NOT NULL CHECK (game_votetotal >= 0),
	game_votesum INT4 DEFAULT '0' NOT NULL CHECK (game_votesum >= 0),
	game_download_total INT4 DEFAULT '0' NOT NULL CHECK (game_download_total >= 0),
	game_download INT2 DEFAULT '1' NOT NULL,
	cat_id INT4 DEFAULT '0' NOT NULL CHECK (cat_id >= 0),
	PRIMARY KEY (game_id)
);


/*
	Table: 'phpbb_arcade_game_rating'
*/
CREATE TABLE phpbb_arcade_game_rating (
	game_id INT4 DEFAULT '0' NOT NULL CHECK (game_id >= 0),
	user_id INT4 DEFAULT '0' NOT NULL CHECK (user_id >= 0),
	game_rating INT4 DEFAULT '0' NOT NULL CHECK (game_rating >= 0),
	rating_date INT4 DEFAULT '0' NOT NULL CHECK (rating_date >= 0),
	user_ip varchar(40) DEFAULT '' NOT NULL,
	PRIMARY KEY (game_id, user_id)
);


/*
	Table: 'phpbb_arcade_scores'
*/
CREATE TABLE phpbb_arcade_scores (
	game_id INT4 DEFAULT '0' NOT NULL CHECK (game_id >= 0),
	user_id INT4 DEFAULT '0' NOT NULL CHECK (user_id >= 0),
	score decimal(25,3) DEFAULT '0' NOT NULL,
	comment_text TEXT DEFAULT '' NOT NULL,
	comment_bitfield varchar(255) DEFAULT '' NOT NULL,
	comment_options INT4 DEFAULT '7' NOT NULL CHECK (comment_options >= 0),
	comment_uid varchar(8) DEFAULT '' NOT NULL,
	score_date INT4 DEFAULT '0' NOT NULL CHECK (score_date >= 0),
	total_time INT4 DEFAULT '0' NOT NULL CHECK (total_time >= 0),
	total_plays INT4 DEFAULT '0' NOT NULL CHECK (total_plays >= 0),
	PRIMARY KEY (game_id, user_id)
);


/*
	Table: 'phpbb_arcade_sessions'
*/
CREATE TABLE phpbb_arcade_sessions (
	session_id char(32) DEFAULT '' NOT NULL,
	game_id INT4 DEFAULT '0' NOT NULL CHECK (game_id >= 0),
	user_id INT4 DEFAULT '0' NOT NULL CHECK (user_id >= 0),
	game_type INT2 DEFAULT '0' NOT NULL,
	randchar1 varchar(30) DEFAULT '' NOT NULL,
	randchar2 varchar(30) DEFAULT '' NOT NULL,
	randgid1 varchar(30) DEFAULT '' NOT NULL,
	randgid2 varchar(30) DEFAULT '' NOT NULL,
	start_time INT4 DEFAULT '0' NOT NULL CHECK (start_time >= 0),
	PRIMARY KEY (session_id)
);


/*
	Table: 'phpbb_arcade_download'
*/
CREATE TABLE phpbb_arcade_download (
	user_id INT4 DEFAULT '0' NOT NULL CHECK (user_id >= 0),
	game_id INT4 DEFAULT '0' NOT NULL CHECK (game_id >= 0),
	total INT4 DEFAULT '0' NOT NULL CHECK (total >= 0),
	download_time INT4 DEFAULT '0' NOT NULL CHECK (download_time >= 0),
	PRIMARY KEY (user_id, game_id)
);



COMMIT;