#
# $Id: $
#

BEGIN TRANSACTION;

# Table: 'phpbb_acl_arcade_groups'
CREATE TABLE phpbb_acl_arcade_groups (
	group_id INTEGER UNSIGNED NOT NULL DEFAULT '0',
	cat_id INTEGER UNSIGNED NOT NULL DEFAULT '0',
	auth_option_id INTEGER UNSIGNED NOT NULL DEFAULT '0',
	auth_role_id INTEGER UNSIGNED NOT NULL DEFAULT '0',
	auth_setting tinyint(2) NOT NULL DEFAULT '0'
);

CREATE INDEX phpbb_acl_arcade_groups_group_id ON phpbb_acl_arcade_groups (group_id);
CREATE INDEX phpbb_acl_arcade_groups_auth_opt_id ON phpbb_acl_arcade_groups (auth_option_id);
CREATE INDEX phpbb_acl_arcade_groups_auth_role_id ON phpbb_acl_arcade_groups (auth_role_id);

# Table: 'phpbb_acl_arcade_options'
CREATE TABLE phpbb_acl_arcade_options (
	auth_option_id INTEGER PRIMARY KEY NOT NULL ,
	auth_option varchar(50) NOT NULL DEFAULT '',
	is_global INTEGER UNSIGNED NOT NULL DEFAULT '0',
	is_local INTEGER UNSIGNED NOT NULL DEFAULT '0',
	founder_only INTEGER UNSIGNED NOT NULL DEFAULT '0'
);

CREATE INDEX phpbb_acl_arcade_options_auth_option ON phpbb_acl_arcade_options (auth_option);

# Table: 'phpbb_acl_arcade_roles'
CREATE TABLE phpbb_acl_arcade_roles (
	role_id INTEGER PRIMARY KEY NOT NULL ,
	role_name varchar(255) NOT NULL DEFAULT '',
	role_description text(65535) NOT NULL DEFAULT '',
	role_type varchar(10) NOT NULL DEFAULT '',
	role_order INTEGER UNSIGNED NOT NULL DEFAULT '0'
);

CREATE INDEX phpbb_acl_arcade_roles_role_type ON phpbb_acl_arcade_roles (role_type);
CREATE INDEX phpbb_acl_arcade_roles_role_order ON phpbb_acl_arcade_roles (role_order);

# Table: 'phpbb_acl_arcade_roles_data'
CREATE TABLE phpbb_acl_arcade_roles_data (
	role_id INTEGER UNSIGNED NOT NULL DEFAULT '0',
	auth_option_id INTEGER UNSIGNED NOT NULL DEFAULT '0',
	auth_setting tinyint(2) NOT NULL DEFAULT '0',
	PRIMARY KEY (role_id, auth_option_id)
);

CREATE INDEX phpbb_acl_arcade_roles_data_ath_op_id ON phpbb_acl_arcade_roles_data (auth_option_id);

# Table: 'phpbb_acl_arcade_users'
CREATE TABLE phpbb_acl_arcade_users (
	user_id INTEGER UNSIGNED NOT NULL DEFAULT '0',
	cat_id INTEGER UNSIGNED NOT NULL DEFAULT '0',
	auth_option_id INTEGER UNSIGNED NOT NULL DEFAULT '0',
	auth_role_id INTEGER UNSIGNED NOT NULL DEFAULT '0',
	auth_setting tinyint(2) NOT NULL DEFAULT '0'
);

CREATE INDEX phpbb_acl_arcade_users_user_id ON phpbb_acl_arcade_users (user_id);
CREATE INDEX phpbb_acl_arcade_users_auth_option_id ON phpbb_acl_arcade_users (auth_option_id);
CREATE INDEX phpbb_acl_arcade_users_auth_role_id ON phpbb_acl_arcade_users (auth_role_id);

# Table: 'phpbb_arcade_access'
CREATE TABLE phpbb_arcade_access (
	cat_id INTEGER UNSIGNED NOT NULL DEFAULT '0',
	user_id INTEGER UNSIGNED NOT NULL DEFAULT '0',
	session_id char(32) NOT NULL DEFAULT '',
	PRIMARY KEY (cat_id, user_id, session_id)
);


# Table: 'phpbb_arcade_categories'
CREATE TABLE phpbb_arcade_categories (
	cat_id INTEGER PRIMARY KEY NOT NULL ,
	parent_id INTEGER UNSIGNED NOT NULL DEFAULT '0',
	left_id INTEGER UNSIGNED NOT NULL DEFAULT '0',
	right_id INTEGER UNSIGNED NOT NULL DEFAULT '0',
	cat_parents mediumtext(16777215) NOT NULL DEFAULT '',
	cat_name text(65535) NOT NULL DEFAULT '',
	cat_desc text(65535) NOT NULL DEFAULT '',
	cat_desc_bitfield varchar(255) NOT NULL DEFAULT '',
	cat_desc_options INTEGER UNSIGNED NOT NULL DEFAULT '7',
	cat_desc_uid varchar(8) NOT NULL DEFAULT '',
	cat_link varchar(255) NOT NULL DEFAULT '',
	cat_password varchar(40) NOT NULL DEFAULT '',
	cat_style INTEGER UNSIGNED NOT NULL DEFAULT '0',
	cat_display INTEGER UNSIGNED NOT NULL DEFAULT '0',
	cat_image varchar(255) NOT NULL DEFAULT '',
	cat_rules text(65535) NOT NULL DEFAULT '',
	cat_rules_link varchar(255) NOT NULL DEFAULT '',
	cat_rules_bitfield varchar(255) NOT NULL DEFAULT '',
	cat_rules_options INTEGER UNSIGNED NOT NULL DEFAULT '7',
	cat_rules_uid varchar(8) NOT NULL DEFAULT '',
	cat_games_per_page tinyint(4) NOT NULL DEFAULT '0',
	cat_type tinyint(4) NOT NULL DEFAULT '0',
	cat_status tinyint(4) NOT NULL DEFAULT '0',
	cat_games INTEGER UNSIGNED NOT NULL DEFAULT '0',
	cat_plays INTEGER UNSIGNED NOT NULL DEFAULT '0',
	cat_download tinyint(1) NOT NULL DEFAULT '1',
	cat_last_play_game_id INTEGER UNSIGNED NOT NULL DEFAULT '0',
	cat_last_play_game_name varchar(255) NOT NULL DEFAULT '0',
	cat_last_play_user_id INTEGER UNSIGNED NOT NULL DEFAULT '0',
	cat_last_play_score decimal(25,3) NOT NULL DEFAULT '0',
	cat_last_play_time INTEGER UNSIGNED NOT NULL DEFAULT '0',
	cat_last_game_installdate INTEGER UNSIGNED NOT NULL DEFAULT '0',
	cat_last_play_username varchar(255) NOT NULL DEFAULT '',
	cat_last_play_user_colour varchar(6) NOT NULL DEFAULT '',
	cat_flags tinyint(4) NOT NULL DEFAULT '32',
	display_subcat_list INTEGER UNSIGNED NOT NULL DEFAULT '1',
	display_on_index INTEGER UNSIGNED NOT NULL DEFAULT '1'
);

CREATE INDEX phpbb_arcade_categories_left_right_id ON phpbb_arcade_categories (left_id, right_id);
CREATE INDEX phpbb_arcade_categories_cat_lastplay_game_id ON phpbb_arcade_categories (cat_last_play_game_id);

# Table: 'phpbb_arcade_config'
CREATE TABLE phpbb_arcade_config (
	config_name varchar(255) NOT NULL DEFAULT '',
	config_value varchar(255) NOT NULL DEFAULT '',
	PRIMARY KEY (config_name)
);


# Table: 'phpbb_arcade_errors'
CREATE TABLE phpbb_arcade_errors (
	error_id INTEGER PRIMARY KEY NOT NULL ,
	user_id INTEGER UNSIGNED NOT NULL DEFAULT '0',
	game_id INTEGER UNSIGNED NOT NULL DEFAULT '0',
	score decimal(25,3) NOT NULL DEFAULT '0',
	error_date INTEGER UNSIGNED NOT NULL DEFAULT '0',
	error_type tinyint(1) NOT NULL DEFAULT '0',
	game_type tinyint(1) NOT NULL DEFAULT '0',
	submitted_game_type tinyint(1) NOT NULL DEFAULT '0',
	played_time int(11) NOT NULL DEFAULT '0',
	error_ip varchar(40) NOT NULL DEFAULT ''
);


# Table: 'phpbb_arcade_favorites'
CREATE TABLE phpbb_arcade_favorites (
	user_id INTEGER UNSIGNED NOT NULL DEFAULT '0',
	game_id INTEGER UNSIGNED NOT NULL DEFAULT '0',
	PRIMARY KEY (user_id, game_id)
);


# Table: 'phpbb_arcade_games'
CREATE TABLE phpbb_arcade_games (
	game_id INTEGER PRIMARY KEY NOT NULL ,
	game_image varchar(255) NOT NULL DEFAULT '',
	game_desc text(65535) NOT NULL DEFAULT '',
	game_highscore decimal(25,3) NOT NULL DEFAULT '0',
	game_highdate INTEGER UNSIGNED NOT NULL DEFAULT '0',
	game_highuser INTEGER UNSIGNED NOT NULL DEFAULT '0',
	game_files mediumtext(16777215) NOT NULL DEFAULT '',
	game_name varchar(255) NOT NULL DEFAULT '',
	game_name_clean varchar(255) NOT NULL DEFAULT '',
	game_swf varchar(255) NOT NULL DEFAULT '',
	game_scorevar varchar(255) NOT NULL DEFAULT '',
	game_type tinyint(1) NOT NULL DEFAULT '0',
	game_width INTEGER UNSIGNED NOT NULL DEFAULT '550',
	game_height INTEGER UNSIGNED NOT NULL DEFAULT '380',
	game_installdate INTEGER UNSIGNED NOT NULL DEFAULT '0',
	game_filesize INTEGER UNSIGNED NOT NULL DEFAULT '0',
	game_scoretype tinyint(1) NOT NULL DEFAULT '0',
	game_order INTEGER UNSIGNED NOT NULL DEFAULT '0',
	game_plays INTEGER UNSIGNED NOT NULL DEFAULT '0',
	game_votetotal INTEGER UNSIGNED NOT NULL DEFAULT '0',
	game_votesum INTEGER UNSIGNED NOT NULL DEFAULT '0',
	game_download_total INTEGER UNSIGNED NOT NULL DEFAULT '0',
	game_download tinyint(1) NOT NULL DEFAULT '1',
	cat_id INTEGER UNSIGNED NOT NULL DEFAULT '0'
);


# Table: 'phpbb_arcade_game_rating'
CREATE TABLE phpbb_arcade_game_rating (
	game_id INTEGER UNSIGNED NOT NULL DEFAULT '0',
	user_id INTEGER UNSIGNED NOT NULL DEFAULT '0',
	game_rating INTEGER UNSIGNED NOT NULL DEFAULT '0',
	rating_date INTEGER UNSIGNED NOT NULL DEFAULT '0',
	user_ip varchar(40) NOT NULL DEFAULT '',
	PRIMARY KEY (game_id, user_id)
);


# Table: 'phpbb_arcade_scores'
CREATE TABLE phpbb_arcade_scores (
	game_id INTEGER UNSIGNED NOT NULL DEFAULT '0',
	user_id INTEGER UNSIGNED NOT NULL DEFAULT '0',
	score decimal(25,3) NOT NULL DEFAULT '0',
	comment_text mediumtext(16777215) NOT NULL DEFAULT '',
	comment_bitfield varchar(255) NOT NULL DEFAULT '',
	comment_options INTEGER UNSIGNED NOT NULL DEFAULT '7',
	comment_uid varchar(8) NOT NULL DEFAULT '',
	score_date INTEGER UNSIGNED NOT NULL DEFAULT '0',
	total_time INTEGER UNSIGNED NOT NULL DEFAULT '0',
	total_plays INTEGER UNSIGNED NOT NULL DEFAULT '0',
	PRIMARY KEY (game_id, user_id)
);


# Table: 'phpbb_arcade_sessions'
CREATE TABLE phpbb_arcade_sessions (
	session_id char(32) NOT NULL DEFAULT '',
	game_id INTEGER UNSIGNED NOT NULL DEFAULT '0',
	user_id INTEGER UNSIGNED NOT NULL DEFAULT '0',
	game_type tinyint(1) NOT NULL DEFAULT '0',
	randchar1 varchar(30) NOT NULL DEFAULT '',
	randchar2 varchar(30) NOT NULL DEFAULT '',
	randgid1 varchar(30) NOT NULL DEFAULT '',
	randgid2 varchar(30) NOT NULL DEFAULT '',
	start_time INTEGER UNSIGNED NOT NULL DEFAULT '0',
	PRIMARY KEY (session_id)
);


# Table: 'phpbb_arcade_download'
CREATE TABLE phpbb_arcade_download (
	user_id INTEGER UNSIGNED NOT NULL DEFAULT '0',
	game_id INTEGER UNSIGNED NOT NULL DEFAULT '0',
	total INTEGER UNSIGNED NOT NULL DEFAULT '0',
	download_time INTEGER UNSIGNED NOT NULL DEFAULT '0',
	PRIMARY KEY (user_id, game_id)
);



COMMIT;