<?php
/**
*
* @package arcade
* @version $Id: arcade_class.php 337 2008-08-02 15:20:27Z jrsweets $
* @copyright (c) 2008 http://www.JeffRusso.net
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

/**
* Class for handling arcade
* @package arcade
*/
class arcade
{
	var $config = array();
	var $games = array();
	var $all_games = array();
	var $cats = array();
	var $users = array();
	var $leaders = array();
	var $totals = array();
	var $latest_highscores = array();
	var $most_popular_games = array();
	var $least_popular_games = array();
	var $longest_highscores = array();
	var $most_downloaded_games = array();
	var $least_downloaded_games = array();
	var $newest_games = array();
	var $cookie_gid = '';
	var $cookie_popup = '';
	var $cookie_sid = '';

	/**
	* Constructor used to define constants, add language file
	* and obtain configuration
	*
	* This is done here to minimize the number of core file
	* edits required to run this mod
	*/
	function arcade($in_arcade = true)
	{
		global $config, $user, $cache, $arcade_cache, $auth_arcade, $template, $phpbb_root_path, $phpEx;

		$arcade_cache = new arcade_cache();
		$this->config = $arcade_cache->obtain_arcade_config();
		$this->set_data();

		// Set cookie information
		$this->cookie_gid = $config['cookie_name'] . '_arcade_gid';
		$this->cookie_popup = $config['cookie_name'] . '_arcade_popup';
		$this->cookie_sid = $config['cookie_name'] . '_arcade_sid';

		$template->assign_vars(array(
			'S_IN_ARCADE'			=> $in_arcade,
			'S_ARCADE_DISABLED'		=> ($this->config['arcade_disable']) ? true : false,
			'U_ARCADE' 				=> append_sid("{$phpbb_root_path}arcade.$phpEx"),
			'U_ARCADE_FAV'			=> append_sid("{$phpbb_root_path}arcade.$phpEx", "mode=fav"),
			'U_ARCADE_STATS'		=> append_sid("{$phpbb_root_path}arcade.$phpEx", "mode=stats"),
			'ARCADE_VERSION_INFO' 	=> $this->config['copyright'],
			)
		);
	}

	function set_data($type = false)
	{
		global $arcade_cache, $auth_arcade;

		switch($type)
		{
			case 'stats':
				$this->users = $arcade_cache->obtain_arcade_users();
				$this->most_popular_games = $this->obtain_popular_games('most', $this->config['most_popular']);
				$this->least_popular_games = $this->obtain_popular_games('least', $this->config['least_popular']);
				$this->longest_highscores = $this->obtain_longest_highscores($this->config['longest_held_scores']);
				
				if ($auth_arcade->acl_getc_global('c_download'))
				{				
					$this->most_downloaded_games = $this->obtain_downloaded_games('most', $this->config['most_downloaded']);
					$this->least_downloaded_games = $this->obtain_downloaded_games('least', $this->config['least_downloaded']);
				}
			// No break

			default:
				$this->newest_games = $this->obtain_newest_games($this->config['newest_games']);
				$this->leaders = $arcade_cache->obtain_arcade_leaders(($this->config['arcade_leaders'] > $this->config['arcade_leaders_header']) ? $this->config['arcade_leaders'] : $this->config['arcade_leaders_header']);
				$this->games = $this->obtain_games();
				$this->all_games = $arcade_cache->obtain_arcade_all_games();
				$this->cats = $arcade_cache->obtain_arcade_cats();
				$this->latest_highscores = $this->obtain_latest_highscores($this->config['latest_highscores']);
				$this->totals = $arcade_cache->obtain_arcade_totals();
			break;
		}
		return;
	}

	function get_permissions($type)
	{
		global $auth_arcade;

		return array_unique(array_keys($auth_arcade->acl_getc($type, true)));
	}

	/**
	* Set config value. Creates missing config entry.
	*/
	function set_config($config_name, $config_value)
	{
		global $db, $cache;

		$sql = 'UPDATE ' . ARCADE_CONFIG_TABLE . "
			SET config_value = '" . $db->sql_escape($config_value) . "'
			WHERE config_name = '" . $db->sql_escape($config_name) . "'";
		$db->sql_query($sql);

		if (!$db->sql_affectedrows() && !isset($this->config[$config_name]))
		{
			$sql = 'INSERT INTO ' . ARCADE_CONFIG_TABLE . ' ' . $db->sql_build_array('INSERT', array(
				'config_name'	=> $config_name,
				'config_value'	=> $config_value));
			$db->sql_query($sql);
		}

		$this->config[$config_name] = $config_value;

		// Destroy the cache of the config
		// because the values have changed
		$cache->destroy('_arcade');
	}

	function set_arcade_favorites($mode, $cat_id, $game_id)
	{
		global $db, $phpbb_root_path, $phpEx, $user, $start, $term;

		$page = request_var('page', '');
		if ($mode == 'addfav' &&  $user->data['is_registered'])
		{
			$data = array(
				'user_id'		=> $user->data['user_id'],
				'game_id'		=> $game_id,
			);

			$sql = 'INSERT INTO ' . ARCADE_FAVS_TABLE . ' ' .
				$db->sql_build_array('INSERT', $data);

			$db->sql_query($sql);
		}
		else if ($mode == 'delfav' && $user->data['is_registered'])
		{
			$sql = 'DELETE FROM ' . ARCADE_FAVS_TABLE . '
				WHERE game_id = ' . (int) $game_id . '
				AND user_id = ' . $user->data['user_id'];
			$db->sql_query($sql);
		}

		$start = ($start < 0) ? 0 : $start;
		if ($cat_id > 0)
		{
			redirect(append_sid("{$phpbb_root_path}arcade.$phpEx", "mode=cat&amp;c=$cat_id&amp;start=$start"));
		}
		else if ($page == 'fav')
		{
			redirect(append_sid("{$phpbb_root_path}arcade.$phpEx", "mode=fav&amp;start=$start"));
		}
		else if ($page == 'search')
		{
			redirect(append_sid("{$phpbb_root_path}arcade.$phpEx", "mode=search&amp;term=$term&amp;start=$start"));
		}
		else if ($page == 'play')
		{
			redirect(append_sid("{$phpbb_root_path}arcade.$phpEx", "mode=play&amp;g=$game_id"));
		}
		redirect(append_sid("{$phpbb_root_path}arcade.$phpEx"));
	}

	function set_game_resolution($size, $width, $height)
	{
		// This function if used to quickly recalutate the correct
		// game resolution if the user decides to change it in the
		// arcade_play.php file (Playing games in full window mode).

		$game = array();
		$game['none'] = '';
		$game['width'] = '';
		$game['height'] = '';
		$game['half'] = '';
		$game['threeq'] = '';
		$game['plus25'] = '';
		$game['plus50'] = '';
		$game['plus75'] = '';
		$game['times2'] = '';

		$widthhalf = round($width * .50);
		$width3q = round($width * .75);
		$width1qx = round($width * 1.25);
		$width2qx = round($width * 1.50);
		$width3qx = round($width * 1.75);
		$width2x = round($width * 2);

		$heighthalf = round($height * .50);
		$height3q = round($height * .75);
		$height1qx = round($height * 1.25);
		$height2qx = round($height * 1.50);
		$height3qx = round($height * 1.75);
		$height2x = round($height * 2);

		switch ($size)
		{
			case 'half':
				$game['width'] = $widthhalf;
				$game['height'] = $heighthalf;
			break;
			case 'threeq':
				$game['width'] = $width3q;
				$game['height'] = $height3q;
			break;
			case 'plus25':
				$game['width'] = $width1qx;
				$game['height'] = $height1qx;
			break;
			case 'plus50':
				$game['width'] = $width2qx;
				$game['height'] = $height2qx;
			break;
			case 'plus75':
				$game['width'] = $width3qx;
				$game['height'] = $height3qx;
			break;
			case 'times2':
				$game['width'] = $width2x;
				$game['height'] = $height2x;
			break;
			default:
	        	$size = 'none';
				$game['width'] = $width;
				$game['height'] = $height;
			break;
		}

		$game[$size] = 'selected="selected"';
		$game['resolution_select'] = "<select name='resolution' onchange='this.form.submit();'>
										<option {$game['half']} value='half'>$widthhalf x $heighthalf</option>
										<option {$game['threeq']} value='threeq'>$width3q x $height3q</option>
										<option {$game['none']} value='none'>$width x $height</option>
										<option {$game['plus25']} value='plus25'>$width1qx x $height1qx</option>
										<option {$game['plus50']} value='plus50'>$width2qx x $height2qx</option>
										<option {$game['times2']} value='times2'>$width2x x $height2x</option></select>";

		return $game;
	}

	function set_arcade_cookie($game_id = false, $game_sid = false, $popup = false, $show = false)
	{
		global $user;

		// The cookies is removed once the arcade_score.php output
		// is displayed.  It just used to check whether of not
		// $gen_simple_header is set in arcade_score.php

		// If the function is called with no parameters and show
		// set to true it will output all cookies
		if (!$game_id && !$game_sid && !$popup && $show)
		{
			print_r($_COOKIE);
			return;
		}

		// If the function is not sent anything it removes all its cookies
		if (!$game_id && !$game_sid && !$popup)
		{
			$set_time = time() - 31536000;
			$user->set_cookie('arcade_popup', '', $set_time);
			$user->set_cookie('arcade_gid', '', $set_time);
			$user->set_cookie('arcade_sid', '', $set_time);

			return;
		}

		$set_time = time() + ARCADE_COOKIE_TIME;
		$user->set_cookie('arcade_gid', $game_id, $set_time);
		$user->set_cookie('arcade_sid', $game_sid, $set_time);
		$user->set_cookie('arcade_popup', ($popup) ? true : false, $set_time);

		return;
	}

	function set_path($file, $type = '')
	{
		global $phpbb_root_path, $phpEx;
		$path = remove_extension($file);

		$return = '';
		switch ($type)
		{
			case 'install':
				$return = $phpbb_root_path . $this->config['game_path'] . $path . '/' . $path . '.' . $phpEx;
			break;

			case 'path':
				$return = $phpbb_root_path . $this->config['game_path'] . $path . '/';
			break;

			default:
				$return = $phpbb_root_path . $this->config['game_path'] . $path . '/' . $file;
			break;
		}

		return $return;
	}
	
	function set_filesize($game_id)
	{
		global $db;
		
		$sql = 'SELECT game_id, game_swf, game_files
			FROM ' . ARCADE_GAMES_TABLE . '
			WHERE game_id = ' . (int) $game_id;
		$result = $db->sql_query($sql);
		$row = $db->sql_fetchrow($result);
		$db->sql_freeresult($result);
		
		$game_files_array = array();		
		$game_files_array[] = $this->set_path($row['game_swf']);
		
		$extra_game_files = ($row['game_files'] != '') ? unserialize($row['game_files']) : '';
		if (is_array($extra_game_files))
		{
			foreach ($extra_game_files as $file)
			{
				if (file_exists($file))
				{
					$game_files_array[] = $file;
				}
			}
		}
		
		$filesize = get_filesize($game_files_array);

		if ($filesize)
		{
			$sql = 'UPDATE ' . ARCADE_GAMES_TABLE . '
				SET game_filesize = '. (int) $filesize . '
				WHERE game_id = ' . (int) $game_id;
			$db->sql_query($sql);		
		}
		
		return $filesize;		
	}
	
	function get_img($path, $alt = '')
	{
		$alt = (!empty($alt)) ? 'alt="' . $alt . '" title="' . $alt . '"' : 'alt=""';
		if (list($width, $height) = @getimagesize($path))
		{
			$img = '<img src="' . $path . '" width="' . $width . '" height="' . $height . '" ' . $alt . ' />';
		}
		else
		{
			$img = '<img src="' . $path . '" ' . $alt . ' />';
		}

		return $img;
	}

	function get_user_info($user_id = false)
	{
		global $db, $phpbb_root_path, $phpEx, $user;

		if ($user_id)
		{
			$sql = 'SELECT * FROM ' . USERS_TABLE . '
					WHERE user_id = ' . $user_id;
			$result = $db->sql_query($sql);
			$user_info = $db->sql_fetchrow($result);
			$db->sql_freeresult($result);
		}
		else
		{
			$user_info = $user->data;
		}

		$avatar = ($user->optionget('viewavatars')) ? get_user_avatar($user_info['user_avatar'], $user_info['user_avatar_type'], $user_info['user_avatar_width'], $user_info['user_avatar_height']) : '';
		$rank_title = $rank_img = $rank_img_src = '';
		get_user_rank($user_info['user_rank'], $user_info['user_posts'], $rank_title, $rank_img, $rank_img_src);

		// Calculates the users total number of arcade victories
		$sql = 'SELECT COUNT(game_id) AS total_wins
			FROM ' . ARCADE_GAMES_TABLE . '
			WHERE game_highuser = ' . $user_info['user_id'];

		$result = $db->sql_query($sql);
		$total_wins = $db->sql_fetchfield('total_wins');
		$db->sql_freeresult($result);

		// Total plays and times using Arcade
		$sql = 'SELECT SUM(total_plays) AS games_played, SUM(total_time) AS games_time
			FROM ' . ARCADE_SCORES_TABLE . '
			WHERE user_id = ' . $user_info['user_id'];

		$result = $db->sql_query($sql);
		$row = $db->sql_fetchrow($result);
		$db->sql_freeresult($result);

		$total_plays = $row['games_played'];
		$total_time = $row['games_time'];

		$data = array(
			'user_id'			=> $user_info['user_id'],
			'username'			=> $user_info['username'],
			'user_colour'		=> $user_info['user_colour'],
			'avatar'	 		=> $avatar,
			'rank_title' 		=> $rank_title,
			'rank_image' 		=> $rank_img,
			'rank_image_src'	=> $rank_img_src,
			'total_wins' 		=> $total_wins,
			'total_plays' 		=> $total_plays,
			'total_time' 		=> $total_time,
		);

		return $data;
	}

	/*function get_game_name($game_id)
	{
		//  If the game id is not set then the user doesn't have the permission to see the game
		//  so we return false.
		if (isset($this->games[$game_id]))
		{
			return $this->games[$game_id]['game_name'];
		}

		// $game = $this->get_game_data($game_id);
		return false; // $game['game_name'];
	}*/


	/**
	* Get any field from the games array (games user has permission for) or all games array
	* This can be used to check if a user has permission to view a game by just passing
	* the game id
	* @returns requested field
	*/
	function get_game_field($game_id, $field = '', $mode = '')
	{
		$mode = ($mode != 'all') ? 'games' : 'all_games';
		switch ($mode)
		{
			case 'games':
				if (isset($this->games[$game_id]))
				{
					if ($field == '')
					{
						return true;
					}

					return $this->games[$game_id][$field];
				}

				return false;
			break;

			case 'all_games':
				if (isset($this->all_games[$game_id]))
				{
					if ($field == '')
					{
						return true;
					}

					return $this->all_games[$game_id][$field];
				}

				$game = $this->get_game_data($game_id);
				return ($game && $field != '' && isset($game[$field])) ? $game[$field] : false;
			break;

			default:
				trigger_error('NO_MODE', E_USER_ERROR);
			break;
		}
	}

	/*function get_game_scorevar($game_id)
	{
		if (isset($this->games[$game_id]))
		{
			return $this->games[$game_id]['game_scorevar'];
		}

		$game = $this->get_game_data($game_id);
		return $game['game_scorevar'];
	}*/

	function get_game_data($game_id)
	{
		global $db;

		$sql_array = array(
			'SELECT'	=> 'g.*, c.cat_name, c.cat_desc, c.cat_desc_uid, c.cat_desc_bitfield, c.cat_desc_options, c.parent_id, c.cat_parents, c.left_id, c.right_id, c.cat_download',

			'FROM'		=> array(
				ARCADE_GAMES_TABLE	=> 'g',
			),

			'LEFT_JOIN'	=> array(
				array(
					'FROM'	=> array(ARCADE_CATS_TABLE => 'c'),
					'ON'	=> 'g.cat_id = c.cat_id'
				),
			),

			'WHERE'	=> $db->sql_in_set('game_id', $game_id),
		);

		$sql = $db->sql_build_query('SELECT', $sql_array);
		$result = $db->sql_query($sql);
		if (is_array($game_id))
		{
			$row = $db->sql_fetchrowset($result);
		}
		else
		{
			$row = $db->sql_fetchrow($result);
		}

		$db->sql_freeresult($result);
		
		if (!$row)
		{
			return false;
		}

		return $row;
	}

	function get_cat_name($cat_id)
	{
		if (isset($this->cats[$cat_id]))
		{
			return $this->cats[$cat_id]['cat_name'];
		}

		$cat = $this->get_cat_data($cat_id);
		return $cat['cat_name'];
	}

	function get_cat_data($cat_id)
	{
		global $db;

		$sql_array = array(
			'SELECT'	=> 'c.cat_id, c.cat_name, c.cat_desc, c.cat_desc_uid, c.cat_desc_bitfield, c.cat_desc_options, c.parent_id, c.cat_parents, c.left_id, c.right_id',

			'FROM'		=> array(
				ARCADE_CATS_TABLE	=> 'c',
			),

			'WHERE'	=> $db->sql_in_set('cat_id', $cat_id),
		);

		$sql = $db->sql_build_query('SELECT', $sql_array);
		$result = $db->sql_query($sql);
		if (is_array($cat_id))
		{
			$row = $db->sql_fetchrowset($result);
		}
		else
		{
			$row = $db->sql_fetchrow($result);
		}

		$db->sql_freeresult($result);

		return $row;
	}

	function get_random_game()
	{
		global $db;

		$sql_array = array(
			'SELECT'	=> 'g.game_id',

			'FROM'		=> array(
				ARCADE_GAMES_TABLE	=> 'g',
			),

			'LEFT_JOIN'	=> array(
				array(
					'FROM'	=> array(ARCADE_CATS_TABLE => 'c'),
					'ON'	=> 'g.cat_id = c.cat_id'
				),
			),

			'WHERE'	=> 'c.cat_status <> ' . ITEM_LOCKED . ' AND ' . $db->sql_in_set('c.cat_id', $this->get_permissions('c_play'), false, true),
		);

		$sql = $db->sql_build_query('SELECT', $sql_array);
		$result = $db->sql_query($sql);
		$row = $db->sql_fetchrowset($result);
		$db->sql_freeresult($result);

		return $row[mt_rand(0, sizeof($row) - 1)]['game_id'];
	}

	function get_fav_data($user_id = false)
	{
		global $db, $user;

		if (!$user_id)
		{
			$user_id = $user->data['user_id'];
		}

		$sql_array = array(
			'SELECT'	=> 'g.game_id, g.game_name, g.game_name_clean, g.game_swf, g.game_scorevar, g.cat_id',

			'FROM'		=> array(
				ARCADE_GAMES_TABLE	=> 'g',
			),

			'LEFT_JOIN'	=> array(
				array(
					'FROM'	=> array(ARCADE_FAVS_TABLE => 'f'),
					'ON'	=> 'g.game_id = f.game_id'
				),
			),

			'WHERE'	=> 'f.user_id = ' . (int) $user_id . ' AND ' . $db->sql_in_set('cat_id', $this->get_permissions('c_view'), false, true),

			'ORDER_BY' => 'g.game_name_clean ASC',
		);

		$sql = $db->sql_build_query('SELECT', $sql_array);
		$result = $db->sql_query($sql);
		$fav = array();
		while ($row = $db->sql_fetchrow($result))
		{
			$fav[$row['game_id']] = array(
				'game_id'		=> $row['game_id'],
				'game_name'		=> $row['game_name'],
				'game_swf'		=> $row['game_swf'],
				'game_scorevar'	=> $row['game_scorevar'],
				'cat_id'		=> $row['cat_id'],
			);
		}
		$db->sql_freeresult($result);
		return $fav;
	}

	function get_played_games($user_id = false)
	{
		global $db, $user;

		if (!$this->config['played_games'])
		{
			return array();
		}

		if (!$user_id)
		{
			$user_id = $user->data['user_id'];
		}

		$sql_array = array(
			'SELECT'	=> 'g.game_id, g.game_name',

			'FROM'		=> array(
				ARCADE_SCORES_TABLE	=> 's',
			),

			'LEFT_JOIN'	=> array(
				array(
					'FROM'	=> array(ARCADE_GAMES_TABLE => 'g'),
					'ON'	=> 's.game_id = g.game_id'
				),
			),

			'WHERE'		=> 's.user_id = ' . (int) $user_id,
		);

		$sql = $db->sql_build_query('SELECT', $sql_array);
		$result = $db->sql_query($sql);

		$played = array();
		while ($row = $db->sql_fetchrow($result))
		{
			$played[$row['game_id']] = $row['game_name'];
		}
		$db->sql_freeresult($result);
		return $played;
	}

	// This function displays all the games for download.  One issue occurs though,
	// since the server is connecting to the site to get the list and the server is most likely
	// not ever going to be logged in we must display every game where the category and/or game
	// is able to be downloaded based on the cat_download and game_download setting.  Once the 
	// list is displayed in the ACP the permissions system will take over for downloading.
	function display_download_list()
	{
		global $db, $config;

		echo $config['sitename'] . "\n";

		$methods = array('.tar');
		$available_methods = array('.tar.gz' => 'zlib', '.tar.bz2' => 'bz2', '.zip' => 'zlib');
		foreach ($available_methods as $type => $module)
		{
			if (!@extension_loaded($module))
			{
				continue;
			}

			$methods[] = $type;
		}

		$sql_array = array(
			'SELECT'	=> 'g.game_id, g.game_swf, g.game_name, g.game_name_clean, g.game_desc, g.game_installdate',

			'FROM'		=> array(
				ARCADE_GAMES_TABLE	=> 'g',
			),

			'LEFT_JOIN'	=> array(
				array(
					'FROM'	=> array(ARCADE_CATS_TABLE => 'c'),
					'ON'	=> 'g.cat_id = c.cat_id'
				),
			),

			'WHERE'	=> 'c.cat_download = 1 AND g.game_download = 1',

			'ORDER_BY' => 'g.game_name_clean ASC',
		);

		$sql = $db->sql_build_query('SELECT', $sql_array);
		$result = $db->sql_query($sql, 3600);
		$row = $db->sql_fetchrowset($result);
		$db->sql_freeresult($result);

		$output = '';
		foreach ($methods as $ext)
		{
			$output .= $ext . '|||';
		}
		$output = substr($output, 0, -3);
		echo $output . "\n";

		foreach ($row as $game_data)
		{
			echo $game_data['game_id'] . '|||' . $game_data['game_swf'] . '|||' . $game_data['game_name'] . '|||' . $game_data['game_name_clean'] . '|||' . str_replace(str_split("\t\n\r"), '', $game_data['game_desc']) . '|||' . $game_data['game_installdate'] . "\n";
		}
		exit;
	}

	function update_download_total($game_id)
	{
		global $db, $user;

		// Update download count
		$sql = 'UPDATE ' . ARCADE_GAMES_TABLE . '
			SET game_download_total = game_download_total + 1
			WHERE game_id = ' . (int) $game_id;
		$db->sql_query($sql);

		$sql = 'UPDATE ' . ARCADE_DOWNLOAD_TABLE. '
			SET total = total + 1, download_time = ' . time() . '
			WHERE game_id = ' . (int) $game_id . ' AND user_id = ' . (int) $user->data['user_id'];
		$db->sql_query($sql);

		if (!$db->sql_affectedrows())
		{
			$sql = 'INSERT INTO ' . ARCADE_DOWNLOAD_TABLE . ' ' . $db->sql_build_array('INSERT', array(
				'game_id'			=> (int) $game_id,
				'user_id'			=> (int) $user->data['user_id'],
				'total'				=> 1,
				'download_time'		=> time(),
			));
			$db->sql_query($sql);
		}
	}

	function time_format($secs)
	{
		// This function is used to convert time in seconds to
		// to Days->Hours->Mins->Secs.

		// Make sure that you include the arcade language file before
		// this functions_arcade.php file or you will have problems
		// displaying time.
		global $user;

		$hms = '';

		$days = intval(intval($secs) / 86400);
		if ($days != 0)
		{
			$secs = $secs - ($days * 86400);
			if ($days == 1)
			{
				$hms .= ' ' . $days . ' ' . $user->lang['ARCADE_DAY'];
			}
			else
			{
				$hms .= ' ' . $days . ' ' . $user->lang['ARCADE_DAYS'];
			}
		}

		$hours = intval(intval($secs) / 3600);
		if ($hours != 0)
		{
			if ($hours == 1)
			{
				$hms .= ' ' . $hours . ' ' . $user->lang['ARCADE_HOUR'];
			}
			else
			{
				$hms .= ' ' . $hours . ' ' . $user->lang['ARCADE_HOURS'];
			}
		}

		$minutes = intval(($secs / 60) % 60);
		if ($minutes != 0)
		{
			if ($minutes == 1)
			{
				$hms .= ' ' . $minutes . ' ' . $user->lang['ARCADE_MIN'];
			}
			else
			{
				$hms .= ' ' . $minutes . ' ' . $user->lang['ARCADE_MINS'];
			}
		}

		$seconds = intval($secs % 60);
		if ($seconds != 0)
		{
			if ($seconds == 1)
			{
				$hms .= ' ' . $seconds . ' ' . $user->lang['ARCADE_SEC'];
			}
			else
			{
				$hms .= ' ' . $seconds . ' ' . $user->lang['ARCADE_SECS'];
			}
		}
		return $hms;
	}	

	function number_format($num)
	{
		global $user;
		
		// This function will format the the passed number to be displayed by the arcade.
		// This will fix the problem that was happening with some games
		// using decimal numbers for scores.  It will only display decimal
		// places if necessary.

		// Anytime that a number is displayed in the arcade for any reason it should
		// go thru this function.  This includes portal blocks and such...
		
		// Lets see if the number has a decimal point
		$decimals = explode('.', $num);
		if (isset($decimals[1]))
		{
			// Ok so we have a decimal point remove all trailing zeroes and get the length
			$decimals = strlen(rtrim($decimals[1], '0'));
		}
		else
		{
			// No decimal point so set it to zero
			$decimals = 0;
		}
		
		return number_format($num, $decimals, $user->lang['ARCADE_DECIMAL_SEP'], $user->lang['ARCADE_THOUSANDS_SEP']);		
	}

	function get_username_string($mode, $user_id, $username, $user_colour)
	{
		global $phpbb_root_path, $phpEx;

		return get_username_string($mode, $user_id, $username, $user_colour, false, append_sid("{$phpbb_root_path}arcade.$phpEx", 'mode=stats'));

	}

	function sync($mode, $where_ids = '')
	{
		global $db, $user;

		if ($where_ids == '' && ($mode == 'rating' || $mode == 'game'))
		{
			trigger_error('ARCADE_SYNC_MODE_NOT_SUPPORTED');
		}

		switch($mode)
		{
			case 'category':
				$sql = 'SELECT cat_id
					FROM ' . ARCADE_CATS_TABLE;

				if ($where_ids != '')
				{
					if (!is_array($where_ids))
					{
						$where_ids = array((int) $where_ids);
						$sql .= ' WHERE ' . $db->sql_in_set('cat_id', $where_ids);
					}
					else
					{
						$sql .= ' WHERE ' . $db->sql_in_set('cat_id', $where_ids) . '
							GROUP BY cat_id';
					}
				}

				$cat_result = $db->sql_query($sql);

				while ($cat_row = $db->sql_fetchrow($cat_result))
				{
					$cat_id = $cat_row['cat_id'];
					$sql = "SELECT COUNT(*) AS cat_games
						FROM " . ARCADE_GAMES_TABLE . "
						WHERE cat_id = $cat_id";

					$result = $db->sql_query($sql);
					$row = $db->sql_fetchrow($result);
					$db->sql_freeresult($result);

					$cat_games = $row['cat_games'];

					$sql_array = array(
						'SELECT'	=> 'SUM(s.total_plays) AS cat_plays',

						'FROM'		=> array(
							ARCADE_SCORES_TABLE	=> 's',
						),

						'LEFT_JOIN'	=> array(
							array(
								'FROM'	=> array(ARCADE_GAMES_TABLE => 'g'),
								'ON'	=> 's.game_id = g.game_id'
							),
						),

						'WHERE'	=> 'g.cat_id = ' . $cat_id,
					);

					$sql = $db->sql_build_query('SELECT', $sql_array);
					$result = $db->sql_query($sql);
					$row = $db->sql_fetchrow($result);
					$db->sql_freeresult($result);

					$cat_plays = ($row['cat_plays'] == '') ? 0 : $row['cat_plays'];

					$sql = "UPDATE " . ARCADE_CATS_TABLE . "
						SET cat_games = $cat_games, cat_plays = $cat_plays
						WHERE cat_id = $cat_id";

					$db->sql_query($sql);
				}
				$db->sql_freeresult($cat_result);
			break;

			case 'game':
				if (!is_array($where_ids))
				{
					$where_ids = ($where_ids) ? array((int) $where_ids) : array();
				}

				$sql = 'SELECT SUM(total_plays) AS game_plays, game_id
					FROM ' . ARCADE_SCORES_TABLE . '
					WHERE ' . $db->sql_in_set('game_id', $where_ids) . '
					GROUP BY game_id';

				$result = $db->sql_query($sql);

				$plays_id = array();
				while ($row = $db->sql_fetchrow($result))
				{
					$plays_id[] = $row['game_id'];
					$sql = "UPDATE " . ARCADE_GAMES_TABLE . "
						SET game_plays = " . $row['game_plays'] . "
						WHERE game_id = " . $row['game_id'];
					$db->sql_query($sql);
				}
				$db->sql_freeresult($result);

				$no_plays = array();
				if (!empty($plays_id))
				{
					foreach ($where_ids as $game_id)
					{
						if (!in_array($game_id, $plays_id))
						{
							$no_plays[] = $game_id;
						}
					}
				}
				else
				{
					$no_plays = $where_ids;
				}

				if (!empty($no_plays))
				{
					$sql = 'UPDATE ' . ARCADE_GAMES_TABLE . '
						SET game_plays = 0
						WHERE ' . $db->sql_in_set('game_id', $no_plays);
					$db->sql_query($sql);
				}
			break;

			case 'rating':
				if (!is_array($where_ids))
				{
					$where_ids = ($where_ids) ? array((int) $where_ids) : array();
				}


				$sql = 'SELECT game_id, COUNT(*) as game_votetotal, SUM(game_rating) as game_votesum
					FROM ' . ARCADE_RATING_TABLE . '
					WHERE ' . $db->sql_in_set('game_id', $where_ids) . '
					GROUP BY game_id';

				$result = $db->sql_query($sql);

				$plays_id = array();
				while ($row = $db->sql_fetchrow($result))
				{
					$plays_id[] = $row['game_id'];
					$returned_results = true;
					$sql = 'UPDATE ' . ARCADE_GAMES_TABLE . '
						SET game_votetotal = ' . $row['game_votetotal'] . ',
							game_votesum = ' . $row['game_votesum'] . '
						WHERE game_id = ' . $row['game_id'];

					$db->sql_query($sql);
				}
				$db->sql_freeresult($result);

				$no_plays = array();
				if (!empty($plays_id))
				{
					foreach ($where_ids as $game_id)
					{
						if (!in_array($game_id, $plays_id))
						{
							$no_plays[] = $game_id;
						}
					}
				}
				else
				{
					$no_plays = $where_ids;
				}

				if (!empty($no_plays))
				{
					$sql = 'UPDATE ' . ARCADE_GAMES_TABLE . '
						SET game_votetotal = 0, game_votesum = 0
						WHERE ' . $db->sql_in_set('game_id', $no_plays);
					$db->sql_query($sql);
				}
			break;

			default:
				trigger_error('NO_MODE');
			break;
		}
		return;
	}

	/**
	* Gets a total number of games based on the parameters it
	* was sent.
	*
	* $type can be set to fav, games or search
	*/
	function get_total($type = '', $id = 0)
	{
		global $db, $user;

		$type = strtolower($type);
		$sql_where = '';

		switch ($type)
		{
			case 'fav':
				if ($id)
				{
					$sql_array = array(
						'SELECT'	=> 'COUNT(f.game_id) as total',

						'FROM'		=> array(
							ARCADE_FAVS_TABLE	=> 'f',
						),

						'LEFT_JOIN'	=> array(
							array(
								'FROM'	=> array(ARCADE_GAMES_TABLE => 'g'),
								'ON'	=> 'f.game_id = g.game_id'
							),
						),

						'WHERE'		=> 'f.user_id = ' . $id . ' AND ' . $db->sql_in_set('g.cat_id', $this->get_permissions('c_view'), false, true),
					);
					$sql = $db->sql_build_query('SELECT', $sql_array);
				}
				else
				{
					trigger_error('ARCADE_NO_ID_ERROR');
				}
			break;

			case 'games':
				$sql_array = array(
					'SELECT'	=> 'COUNT(game_id) as total',

					'FROM'		=> array(
						ARCADE_GAMES_TABLE	=> 'g',
					),
				);

				if ($id)
				{
					$sql_array['WHERE'] = 'cat_id = ' . $id;
				}
				$sql = $db->sql_build_query('SELECT', $sql_array);
			break;

			case 'scores':

				if ($id)
				{
					$sql_array = array(
						'SELECT'	=> 'COUNT(score) as total',

						'FROM'		=> array(
							ARCADE_SCORES_TABLE	=> 's',
						),

						'LEFT_JOIN'	=> array(
							array(
								'FROM'	=> array(USERS_TABLE => 'u'),
								'ON'	=> 's.user_id = u.user_id'
							),
						),

						'WHERE'		=> 's.game_id = ' . $id,
					);
					$sql = $db->sql_build_query('SELECT', $sql_array);
				}
				else
				{
					$sql_array = array(
						'SELECT'	=> 'g.game_id',

						'FROM'		=> array(
							ARCADE_SCORES_TABLE	=> 's',
						),

						'LEFT_JOIN'	=> array(
							array(
								'FROM'	=> array(ARCADE_GAMES_TABLE => 'g'),
								'ON'	=> 'g.game_id = s.game_id'
							),
						),
					);

					$sql = $db->sql_build_query('SELECT_DISTINCT', $sql_array);
					$result = $db->sql_query($sql);
					$row = $db->sql_fetchrowset($result);
					$db->sql_freeresult($result);

					return sizeof($row);
				}
			break;

			case 'search':
				if ($id)
				{
					$sql_array = array(
						'SELECT'	=> 'COUNT(g.game_id) as total',

						'FROM'		=> array(
							ARCADE_GAMES_TABLE	=> 'g',
						),

						'WHERE'		=> '(g.game_name_clean '. $db->sql_like_expression($id) . '
											OR LOWER(g.game_desc) ' . $db->sql_like_expression($id) . ')
											AND ' . $db->sql_in_set('g.cat_id', $this->get_permissions('c_view'), false, true),
					);
					$sql = $db->sql_build_query('SELECT', $sql_array);
				}
				else
				{
					trigger_error('ARCADE_NO_ID_ERROR');
				}
			break;

			case 'download_stats':
				if ($id)
				{
					$sql_array = array(
						'SELECT'	=> 'COUNT(game_id) as total',

						'FROM'		=> array(
							ARCADE_DOWNLOAD_TABLE	=> 'd',
						),

						'WHERE'		=> 'user_id = ' . (int) $id,
					);
					$sql = $db->sql_build_query('SELECT', $sql_array);
				}
				else
				{
					$sql_array = array(
						'SELECT'	=> 'user_id',

						'FROM'		=> array(
							ARCADE_DOWNLOAD_TABLE	=> 'd',
						),

						'GROUP_BY'		=> 'user_id',
					);
					$sql = $db->sql_build_query('SELECT', $sql_array);

					$result = $db->sql_query($sql);
					$row = $db->sql_fetchrowset($result);
					$db->sql_freeresult($result);

					return sizeof($row);
				}
			break;

			default:
				trigger_error('ARCADE_NO_TOTAL_TYPE_ERROR');
			break;
		}

		$result = $db->sql_query($sql);

		$row = $db->sql_fetchrow($result);
		$db->sql_freeresult($result);

		return $row['total'];
	}

	/*
	This is never used for IBProV3 or IBPro arcadelib games because it will cause scoring errors with these types.
	If a game is having scoring errors, try shutting this off for the specific game type.
	*/
	function get_protection($type)
	{
		$use_protection = false;
		switch($type)
		{
			case AMOD_GAME:
				$use_protection = ($this->config['protect_amod']) ? true : false;
			break;

			case IBPRO_GAME:
				$use_protection = ($this->config['protect_ibpro']) ? true : false;
			break;

			case V3ARCADE_GAME:
				$use_protection = ($this->config['protect_v3arcade']) ? true : false;
			break;

			default:
			break;
		}

		return $use_protection;
	}

	function set_rating_image($votes, $sum, $avg_score)
	{
		global $user;

		$image = '';
		$rating_score = round($avg_score);

		$votes = ($votes != 1 ) ? $votes . ' ' . $user->lang['ARCADE_VOTES'] : $votes . ' ' . $user->lang['ARCADE_VOTE'];
		for ($i = 1; $i <= $rating_score; $i++)
		{
			$image .= '<img src="./' . $this->config['image_path'] . 'star1.gif" title="' . $votes . '" alt="' . $votes . '" />';
		}
		$blank_stars = ( 5 - $rating_score);
		for ($i = 1; $i <= $blank_stars; $i++)
		{
			$image .= '<img src="./' . $this->config['image_path'] . 'star2.gif" title="' . $votes . '" alt="' . $votes . '" />';
		}

		return $image;
	}

	function set_fav_image($game_data, $game_id, $cat_id)
	{
		global $user, $phpbb_root_path, $phpEx, $start, $term, $userdata;

		if (!$user->data['is_registered'] || $user->data['is_bot'])
		{
			return;
		}
		else
		{
			$start = ($start < 0) ? 0 : $start;
			if ($cat_id > 0)
			{
				$locate = '&amp;c=' . $cat_id;
			}
			else if ($cat_id == ARCADE_FAV_ID)
			{
				$locate = '&amp;page=fav';
			}
			else if ($cat_id == ARCADE_SEARCH_ID)
			{
				$locate = '&amp;page=search&amp;term=' . $term;
			}
			else if ($cat_id == ARCADE_PLAY_ID)
			{
				$locate = '&amp;page=play';
			}


			$image = '<a href="' . append_sid("{$phpbb_root_path}arcade.$phpEx", "mode=addfav$locate&amp;g=$game_id&amp;start=$start") . '"><img src="./' . $this->config['image_path'] . 'favorite.gif" border="0" title="'.$user->lang['ARCADE_ADD_FAV'].'" alt="'.$user->lang['ARCADE_ADD_FAV'].'" /></a>';

			if (isset($game_data[$game_id]))
			{
				// Since we got a match lets break from the loop...
				$image = '<a href="' . append_sid("{$phpbb_root_path}arcade.$phpEx", "mode=delfav$locate&amp;g=$game_id&amp;start=$start") . '"><img src="./' . $this->config['image_path'] . 'remove_favorite.gif" border="0" title="'.$user->lang['ARCADE_REMOVE_FAV'].'" alt="'.$user->lang['ARCADE_REMOVE_FAV'].'" /></a>';
			}

			return $image;
		}
	}

	function get_rating_select($user_choice)
	{
		global $user;

		$selected = array();
		for ($i = 0; $i <= 5; $i++)
		{
			if ($user_choice == $i)
			{
				$selected[$i] = 'selected';
			}
			else
			{
				$selected[$i] = '';
			}
		}

		$rating_select = '<select name="game_rating">
							<option value="0" ' . $selected[0] . '>' . $user->lang['ARCADE_SELECT_RATING'] . '</option>
							<option value="1" ' . $selected[1] . '>1 - ' . $user->lang['ARCADE_RATING_BAD'] . '</option>
							<option value="2" ' . $selected[2] . '>2</option>
							<option value="3" ' . $selected[3] . '>3 - ' . $user->lang['ARCADE_RATING_AVG'] . '</option>
							<option value="4" ' . $selected[4] . '>4</option>
							<option value="5" ' . $selected[5] . '>5 - ' . $user->lang['ARCADE_RATING_GREAT'] . '</option>
						</select>';

		return $rating_select;
	}

	function prepare_score($game_sid, $game_scorevar, $game_type)
	{
		// This function checks to make sure the game type is the expected type
		// and that the user and game ids exist in the sessions table.
		// It then returns the needed information in an array.
		global $db, $user, $score;

		$sql_array = array(
			'SELECT'	=> 'g.game_id, g.game_name, g.game_highscore, g.game_highuser, g.game_scoretype, g.game_type, g.cat_id, s.session_id, s.start_time, u.username, c.cat_name, c.cat_desc, c.cat_desc_uid, c.cat_desc_bitfield, c.cat_desc_options, c.parent_id, c.left_id, c.right_id, c.cat_parents',

			'FROM'		=> array(
				ARCADE_GAMES_TABLE	=> 'g',
			),

			'LEFT_JOIN'	=> array(
				array(
					'FROM'	=> array(ARCADE_CATS_TABLE => 'c'),
					'ON'	=> 'g.cat_id = c.cat_id'
				),
				array(
					'FROM'	=> array(ARCADE_SESSIONS_TABLE => 's'),
					'ON'	=> 'g.game_id = s.game_id'
				),
				array(
					'FROM'	=> array(USERS_TABLE => 'u'),
					'ON'	=> 'g.game_highuser = u.user_id'
				),
			),
		);

		if (is_int($game_scorevar))
		{
			$sql_array['WHERE']	= 'g.game_id = ' . (int) $game_scorevar . '
					AND s.user_id = ' . $user->data['user_id'];
		}
		else
		{
			$sql_array['WHERE']	= "g.game_scorevar = '$game_scorevar'
					AND s.user_id = " . $user->data['user_id'];
		}

		$sql = $db->sql_build_query('SELECT', $sql_array);
		$result = $db->sql_query($sql);

		$row = $db->sql_fetchrow($result);
		$db->sql_freeresult($result);

		if (!$row)
		{
			trigger_error('ARCADE_BACK_BUTTON_ERROR');
		}

		$row['current_time'] = time();
		$row['total_time'] = $row['current_time'] - $row['start_time'];

		// Remove old entries from Arcade Sessions Table
		$sql = 'DELETE FROM ' . ARCADE_SESSIONS_TABLE . '
			WHERE start_time < ' . (time() - 72000 ) . '
				OR (user_id = ' . $user->data['user_id'] . '
			AND game_id = ' . (int) $row['game_id'] . ')';

		$db->sql_query($sql);

		// This will check somethings to try and verify the integrity of the score
		// If something is in question it will add an entry to the error table which
		// the admin will be able to view in the ACP.  There they can make a decision
		// what to do...

		$game_type_error = false;
		if (!($row['game_type'] == IBPRO_ARCADELIB_GAME && $game_type == IBPRO_GAME) && ($row['game_type'] != $game_type))
		{
			$game_type_error = true;
		}

		if (($row['session_id'] != $game_sid) || $game_type_error || ($row['total_time'] <= 0))
		{
			$error_type = ARCADE_ERROR_UNKNOWN;
			if ($row['session_id'] != $game_sid)
			{
				$error_type = ARCADE_ERROR_SESSION;
			}
			else if ($row['game_type'] != $game_type)
			{
				$error_type = ARCADE_ERROR_GAMETYPE;
			}
			else if ($row['total_time'] <= 0)
			{
				$error_type = ARCADE_ERROR_TIME;
			}

			$sql_ary = array(
				'user_id'				=> $user->data['user_id'],
				'game_id'				=> $row['game_id'],
				'score'					=> $score,
				'error_date'			=> time(),
				'error_type'			=> $error_type,
				'game_type'				=> $row['game_type'],
				'submitted_game_type'	=> $game_type,
				'played_time'			=> $row['total_time'],
				'error_ip'				=> $user->ip,
			);

			$sql = 'INSERT INTO ' . ARCADE_ERRORS_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary);
			$db->sql_query($sql);
		}

		// This is the only one we will stop submission on.  This is because there is no other
		// cause for this to happen other than cheating or admin set game type wrong.  Either
		// way it needs to be fixed.  The others could have a valid reason.
		if ($game_type_error)
		{
			trigger_error('ARCADE_TYPE_ERROR');
		}

		return $row;
	}

	function prepare_pm(&$text, $score_data)
	{
		global $user;

		// Used in conjuction with the send_pm function.  This is
		// used to replace the place holders with the data...
		$text = str_replace('[game_id]', $score_data['game_id'], $text);
		$text = str_replace('[game_name]', $score_data['game_name'], $text);
		$text = str_replace('[old_user_id]', $score_data['old_user_id'], $text);
		$text = str_replace('[old_username]', $score_data['old_username'], $text);
		$text = str_replace('[new_user_id]', $user->data['user_id'], $text);
		$text = str_replace('[new_username]', $user->data['username'], $text);
		$text = str_replace('[old_score]', $score_data['old_score'], $text);
		$text = str_replace('[new_score]', $score_data['new_score'], $text);

		$game_link = '[url=' . generate_board_url() . '/arcade.php?mode=play&g=' . $score_data['game_id'] . ']' . $score_data['game_name'] . '[/url]';
		$username = ($user->data['user_colour']) ? '[color=#' .  $user->data['user_colour'] . ']' . $user->data['username'] . '[/color]' : $user->data['username'];
		$user_link = '[url=' . generate_board_url() . '/arcade.php?mode=stats&u=' . $user->data['user_id'] . ']' . $username . '[/url]';
		$old_user_link = '[url=' . generate_board_url() . '/arcade.php?mode=stats&u=' . $score_data['old_user_id'] . ']' . $user->lang['ARCADE_HERE'] . '[/url]';

		$text = str_replace('[game_link]', $game_link, $text);
		$text = str_replace('[user_link]', $user_link, $text);
		$text = str_replace('[old_user_link]', $old_user_link, $text);

		return;
	}

	function send_pm($subject, $message, $score_data)
	{
		global $user, $phpbb_root_path, $phpEx;

		if (!$user->data['is_registered'])
		{
			return;
		}

		include($phpbb_root_path . 'includes/functions_privmsgs.' . $phpEx);
		include($phpbb_root_path . 'includes/message_parser.' . $phpEx);

		$message_parser = new parse_message();

		$this->prepare_pm($subject, $score_data);
		$this->prepare_pm($message, $score_data);

		$subject = utf8_normalize_nfc($subject);
		$message_parser->message = utf8_normalize_nfc($message);
		$address_list['u'][$score_data['old_user_id']] = 'to';

		$message_parser->parse(true, true, true);

		$pm_data = array(
					'msg_id'				=> 0,
					'from_user_id'			=> $user->data['user_id'],
					'from_user_ip'			=> $user->data['user_ip'],
					'from_username'			=> $user->data['username'],
					'reply_from_root_level'	=> 0,
					'reply_from_msg_id'		=> 0,
					'icon_id'				=> 0,
					'enable_sig'			=> true,
					'enable_bbcode'			=> true,
					'enable_smilies'		=> true,
					'enable_urls'			=> true,
					'bbcode_bitfield'		=> $message_parser->bbcode_bitfield,
					'bbcode_uid'			=> $message_parser->bbcode_uid,
					'message'				=> $message_parser->message,
					'attachment_data'		=> $message_parser->attachment_data,
					'filename_data'			=> $message_parser->filename_data,
					'address_list'			=> $address_list
		);
		unset($message_parser);
		submit_pm('post', $subject, $pm_data, true);

		return;
	}

	function limit_play($cat_id)
	{
		global $user, $db, $auth_arcade;

		if ($auth_arcade->acl_get('c_ignorecontrol', $cat_id))
		{
			return;
		}	

		if ($this->config['limit_play'] == LIMIT_PLAY_TYPE_POSTS || $this->config['limit_play'] == LIMIT_PLAY_TYPE_DAYS || $this->config['limit_play'] == LIMIT_PLAY_TYPE_BOTH)
		{		
			$total_posts = $this->config['limit_play_total_posts'];
			$days = $this->config['limit_play_days'];
			$posts = $this->config['limit_play_posts'];
			$current_time = time();
			$old_time = $current_time - (86400 * $days);
			$error = array();

			if ($this->config['limit_play'] == LIMIT_PLAY_TYPE_POSTS || $this->config['limit_play'] == LIMIT_PLAY_TYPE_BOTH)
			{
				$sql = 'SELECT COUNT(post_id) as total_posts
						FROM ' . POSTS_TABLE . '
						WHERE poster_id = ' . $user->data['user_id'];
				
				$result = $db->sql_query($sql);
				$actual_total_posts = $db->sql_fetchfield('total_posts');
				$db->sql_freeresult($result);
				
				if ($actual_total_posts < $total_posts)
				{
					$error[] = sprintf($user->lang['ARCADE_LIMIT_PLAY_TYPE_POSTS'], $total_posts,  $total_posts - $actual_total_posts);
				}
			}			
			
			if  ($this->config['limit_play'] == LIMIT_PLAY_TYPE_DAYS || $this->config['limit_play'] == LIMIT_PLAY_TYPE_BOTH)
			{
				$sql = 'SELECT COUNT(post_id) as total_posts
						FROM ' . POSTS_TABLE . '
						WHERE poster_id = ' . $user->data['user_id'] . '
							AND post_time
						BETWEEN ' . $old_time . ' AND ' . $current_time;
				
				$result = $db->sql_query($sql);
				$actual_posts_per_day = $db->sql_fetchfield('total_posts');
				$db->sql_freeresult($result);
				
				if ($actual_posts_per_day < $posts)
				{
					$error[] = sprintf($user->lang['ARCADE_LIMIT_PLAY_TYPE_DAYS'], $posts, $days, $posts - $actual_posts_per_day, $days);
				}
			}
			
			if (sizeof($error))
			{
				trigger_error(implode('<br /><br />', $error));
			}

		}
	}

	function debug($exit = false, $var = false)
	{
		if ($var)
		{
			print '<pre>';
			var_export($var);
			print '</pre>';
		}
		else
		{
			print '<pre>';
			if (is_array($_POST))
			{
				echo var_export($_POST, true);
			}
			else
			{
				print $_POST;
			}
			print '</pre>';
		}

		if ($exit)
		{
			exit;
		}
	}

	function set_header_no_cache()
	{
		@header("HTTP/1.0 200 OK");
		@header("HTTP/1.1 200 OK");
		@header("Content-type: text/html");
		@header("Cache-Control: no-store,no-cache, must-revalidate, max-age=0");
		@header("Expires: Mon, 27 Jul 1981 05:00:00 GMT");
		@header("pragma: no-cache");
	}

	function create_session($game_id, $game_type)
	{
		global $db, $user;

		// Delete old and/or duplicate arcade sessions...
		$sql = 'DELETE FROM ' . ARCADE_SESSIONS_TABLE . '
				WHERE start_time < ' . (time() - 72000 ) . '
					OR (user_id = ' . $user->data['user_id'] . '
						AND game_id = ' . (int) $game_id . ')';

		if ($user->data['user_id'] != ANONYMOUS)
		{
			$sql .= ' OR user_id = ' . $user->data['user_id'];
		}
		$db->sql_query($sql);

		$game_sid = md5(uniqid($user->ip));

		$randomgid1 = rand(1, 200);
		$randomgid2 = rand(1, 200);
		$gidencoded = $game_id * $randomgid1 ^ $randomgid2;

		$sql_ary = array(
			'session_id'		=> $game_sid,
			'game_id'			=> (int) $game_id,
			'user_id'			=> (int) $user->data['user_id'],
			'game_type'			=> $game_type,
			'randgid1'			=> $randomgid1,
			'randgid2'			=> $randomgid2,
			'start_time'		=> time(),
		);

		$sql = 'INSERT INTO ' . ARCADE_SESSIONS_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary);
		$db->sql_query($sql);

		$data = array(
			'game_sid' 			=> $game_sid,
			'gidencoded'		=> $gidencoded,
		);

		return $data;
	}

	function compress_methods($remove_dot = false)
	{
		if ($remove_dot)
		{
			$methods = array('tar');
			$available_methods = array('gz' => 'zlib', 'bz2' => 'bz2', 'zip' => 'zlib');
		}
		else
		{
			$methods = array('.tar');
			$available_methods = array('.tar.gz' => 'zlib', '.tar.bz2' => 'bz2', '.zip' => 'zlib');
		}

		foreach ($available_methods as $type => $module)
		{
			if (!@extension_loaded($module))
			{
				continue;
			}
			$methods[] = $type;
		}

		return $methods;
	}

	function obtain_latest_highscores($limit)
	{
		global $db;

		$sql_array = array(
			'SELECT'	=> 'g.game_id, g.game_name, g.game_highuser, g.game_highscore, g.game_highdate, g.cat_id, u.username, u.user_colour',

			'FROM'		=> array(
				ARCADE_GAMES_TABLE	=> 'g',
			),

			'LEFT_JOIN'	=> array(
				array(
					'FROM'	=> array(USERS_TABLE => 'u'),
					'ON'	=> 'g.game_highuser = u.user_id'
				)
			),

			'WHERE'		=> 'g.game_highscore > 0 AND ' . $db->sql_in_set('cat_id', $this->get_permissions('c_view'), false, true),

			'ORDER_BY'	=> 'g.game_highdate DESC'
		);

		$sql = $db->sql_build_query('SELECT', $sql_array);
		$result = $db->sql_query_limit($sql, $limit, 0, $this->config['cache_time'] * 3600);

		$row = $db->sql_fetchrowset($result);
		$db->sql_freeresult($result);

		return $row;
	}

	function obtain_newest_games($limit)
	{
		global $db, $arcade;

		$sql = 'SELECT game_id, game_image, game_name, game_scorevar, game_installdate, cat_id
			FROM ' . ARCADE_GAMES_TABLE . '
			WHERE ' . $db->sql_in_set('cat_id', $this->get_permissions('c_view'), false, true) . '
			ORDER BY game_installdate DESC';

		$result = $db->sql_query_limit($sql, $limit, 0, $this->config['cache_time'] * 3600);

		$row = $db->sql_fetchrowset($result);
		$db->sql_freeresult($result);

		return $row;
	}
	
	function obtain_downloaded_games($type, $limit)
	{
		global $db, $arcade;
		
		$type = strtolower($type);
		switch ($type)
		{
			case 'most':
				$order = 'DESC';
			break;

			case 'least':
				$order = 'ASC';
			break;

			default:
				trigger_error('ARCADE_NO_ORDER_TYPE_ERROR');
			break;
		}

		$sql = 'SELECT game_id, game_image, game_name, game_scorevar, game_installdate, game_download_total, cat_id
			FROM ' . ARCADE_GAMES_TABLE . '
			WHERE ' . $db->sql_in_set('cat_id', $this->get_permissions('c_view'), false, true) . '
			ORDER BY game_download_total ' . $order;

		$result = $db->sql_query_limit($sql, $limit, 0, $this->config['cache_time'] * 3600);

		$row = $db->sql_fetchrowset($result);
		$db->sql_freeresult($result);

		return $row;
	}

	function obtain_popular_games($type, $limit)
	{
		global $db, $arcade;

		$type = strtolower($type);
		switch ($type)
		{
			case 'most':
				$order = 'DESC';
			break;

			case 'least':
				$order = 'ASC';
			break;

			default:
				trigger_error('ARCADE_NO_ORDER_TYPE_ERROR');
			break;
		}

		$sql = 'SELECT game_id, game_name, game_image, game_plays, game_scorevar, cat_id
			FROM ' . ARCADE_GAMES_TABLE . '
			WHERE ' . $db->sql_in_set('cat_id', $this->get_permissions('c_view'), false, true) . '
				AND game_plays > 0
			ORDER BY game_plays ' . $order;

		$result = $db->sql_query_limit($sql, $limit, 0, $this->config['cache_time'] * 3600);

		$row = $db->sql_fetchrowset($result);
		$db->sql_freeresult($result);

		return $row;
	}

	function obtain_longest_highscores($limit)
	{
		global $db, $arcade;

		$sql_array = array(
			'SELECT'	=> 'g.game_id, g.game_name, g.game_highscore, g.game_highdate, g.cat_id, u.username, u.user_id, u.user_colour',

			'FROM'		=> array(
				ARCADE_GAMES_TABLE	=> 'g',
			),

			'LEFT_JOIN'	=> array(
				array(
					'FROM'	=> array(USERS_TABLE => 'u'),
					'ON'	=> 'g.game_highuser = u.user_id'
				)
			),

			'WHERE'		=> 'g.game_highscore > 0 AND ' . $db->sql_in_set('cat_id', $this->get_permissions('c_view'), false, true),

			'ORDER_BY'	=> 'g.game_highdate ASC'
		);

		$sql = $db->sql_build_query('SELECT', $sql_array);
		$result = $db->sql_query_limit($sql, $limit, 0, $this->config['cache_time'] * 3600);

		$row = $db->sql_fetchrowset($result);
		$db->sql_freeresult($result);

		return $row;
	}

	/**
	* Obtain arcade games
	*/
	function obtain_games()
	{
		global $db;

		$sql = 'SELECT game_id, game_name, game_name_clean, game_swf, game_scorevar, cat_id
			FROM ' . ARCADE_GAMES_TABLE . '
			WHERE ' . $db->sql_in_set('cat_id', $this->get_permissions('c_view'), false, true) . '
			ORDER BY game_name_clean ASC';
		$result = $db->sql_query($sql, $this->config['cache_time'] * 3600);

		$games = array();
		while ($row = $db->sql_fetchrow($result))
		{
			$games[$row['game_id']] = array(
				'game_id'		=> $row['game_id'],
				'game_name'		=> $row['game_name'],
				'game_swf'		=> $row['game_swf'],
				'game_scorevar'	=> $row['game_scorevar'],
				'cat_id'		=> $row['cat_id'],
			);
		}
		$db->sql_freeresult($result);

		return $games;
	}
}
?>